class ErrorProvider(Component,IComponent,IDisposable,IExtenderProvider,ISupportInitialize):
 """
 Provides a user interface for indicating that a control on a form has an error associated with it.

 

 ErrorProvider()

 ErrorProvider(parentControl: ContainerControl)

 ErrorProvider(container: IContainer)
 """
 def BindToDataAndErrors(self,newDataSource,newDataMember):
  """
  BindToDataAndErrors(self: ErrorProvider,newDataSource: object,newDataMember: str)

   Provides a method to set both the System.Windows.Forms.ErrorProvider.DataSource and 

    System.Windows.Forms.ErrorProvider.DataMember at run time.

  

  

   newDataSource: A data set based on the System.Collections.IList interface to be monitored for errors. 

    Typically,this is a System.Data.DataSet to be monitored for errors.

  

   newDataMember: A collection within the newDataSource to monitor for errors. Typically,this will be a 

    System.Data.DataTable.
  """
  pass
 def CanExtend(self,extendee):
  """
  CanExtend(self: ErrorProvider,extendee: object) -> bool

  

   Gets a value indicating whether a control can be extended.

  

   extendee: The control to be extended.

   Returns: true if the control can be extended; otherwise,false.This property will be true if the object 

    is a System.Windows.Forms.Control and is not a System.Windows.Forms.Form or 

    System.Windows.Forms.ToolBar.
  """
  pass
 def Clear(self):
  """
  Clear(self: ErrorProvider)

   Clears all settings associated with this component.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: ErrorProvider,disposing: bool)

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def GetError(self,control):
  """
  GetError(self: ErrorProvider,control: Control) -> str

  

   Returns the current error description string for the specified control.

  

   control: The item to get the error description string for.

   Returns: The error description string for the specified control.
  """
  pass
 def GetIconAlignment(self,control):
  """
  GetIconAlignment(self: ErrorProvider,control: Control) -> ErrorIconAlignment

  

   Gets a value indicating where the error icon should be placed in relation to the control.

  

   control: The control to get the icon location for.

   Returns: One of the System.Windows.Forms.ErrorIconAlignment values. The default icon alignment is 

    System.Windows.Forms.ErrorIconAlignment.MiddleRight.
  """
  pass
 def GetIconPadding(self,control):
  """
  GetIconPadding(self: ErrorProvider,control: Control) -> int

  

   Returns the amount of extra space to leave next to the error icon.

  

   control: The control to get the padding for.

   Returns: The number of pixels to leave between the icon and the control.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def OnRightToLeftChanged(self,*args):
  """
  OnRightToLeftChanged(self: ErrorProvider,e: EventArgs)

   Raises the System.Windows.Forms.ErrorProvider.RightToLeftChanged event.

  

   e: A System.EventArgs that contains the event data.
  """
  pass
 def SetError(self,control,value):
  """
  SetError(self: ErrorProvider,control: Control,value: str)

   Sets the error description string for the specified control.

  

   control: The control to set the error description string for.

   value: The error description string,or null or System.String.Empty to remove the error.
  """
  pass
 def SetIconAlignment(self,control,value):
  """
  SetIconAlignment(self: ErrorProvider,control: Control,value: ErrorIconAlignment)

   Sets the location where the error icon should be placed in relation to the control.

  

   control: The control to set the icon location for.

   value: One of the System.Windows.Forms.ErrorIconAlignment values.
  """
  pass
 def SetIconPadding(self,control,padding):
  """
  SetIconPadding(self: ErrorProvider,control: Control,padding: int)

   Sets the amount of extra space to leave between the specified control and the error icon.

  

   control: The control to set the padding for.

   padding: The number of pixels to add between the icon and the control.
  """
  pass
 def UpdateBinding(self):
  """
  UpdateBinding(self: ErrorProvider)

   Provides a method to update the bindings of the System.Windows.Forms.ErrorProvider.DataSource,

    System.Windows.Forms.ErrorProvider.DataMember,and the error text.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,parentControl: ContainerControl)

  __new__(cls: type,container: IContainer)
  """
  pass
 def __str__(self,*args):
  pass
 BlinkRate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the rate at which the error icon flashes.



Get: BlinkRate(self: ErrorProvider) -> int



Set: BlinkRate(self: ErrorProvider)=value

"""

 BlinkStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating when the error icon flashes.



Get: BlinkStyle(self: ErrorProvider) -> ErrorBlinkStyle



Set: BlinkStyle(self: ErrorProvider)=value

"""

 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 ContainerControl=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating the parent control for this System.Windows.Forms.ErrorProvider.



Get: ContainerControl(self: ErrorProvider) -> ContainerControl



Set: ContainerControl(self: ErrorProvider)=value

"""

 DataMember=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the list within a data source to monitor.



Get: DataMember(self: ErrorProvider) -> str



Set: DataMember(self: ErrorProvider)=value

"""

 DataSource=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the data source that the System.Windows.Forms.ErrorProvider monitors.



Get: DataSource(self: ErrorProvider) -> object



Set: DataSource(self: ErrorProvider)=value

"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 Icon=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Drawing.Icon that is displayed next to a control when an error description string has been set for the control.



Get: Icon(self: ErrorProvider) -> Icon



Set: Icon(self: ErrorProvider)=value

"""

 RightToLeft=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the component is used in a locale that supports right-to-left fonts.



Get: RightToLeft(self: ErrorProvider) -> bool



Set: RightToLeft(self: ErrorProvider)=value

"""

 Site=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Set: Site(self: ErrorProvider)=value

"""

 Tag=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets an object that contains data about the component.



Get: Tag(self: ErrorProvider) -> object



Set: Tag(self: ErrorProvider)=value

"""


 RightToLeftChanged=None




class CoClassAttribute(Attribute,_Attribute):
 """
 Specifies the class identifier of a coclass imported from a type library.

 

 CoClassAttribute(coClass: Type)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,coClass):
  """ __new__(cls: type,coClass: Type) """
  pass
 CoClass=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the class identifier of the original coclass.



Get: CoClass(self: CoClassAttribute) -> Type



"""





class GridViewColumn(DependencyObject,INotifyPropertyChanged):
 """
 Represents a column that displays data.

 

 GridViewColumn()
 """
 def OnHeaderStringFormatChanged(self,*args):
  """
  OnHeaderStringFormatChanged(self: GridViewColumn,oldHeaderStringFormat: str,newHeaderStringFormat: str)

   Occurs when the System.Windows.Controls.GridViewColumn.HeaderStringFormat property changes.

  

   oldHeaderStringFormat: The old value of the System.Windows.Controls.GridViewColumn.HeaderStringFormat property.

   newHeaderStringFormat: The new value of the System.Windows.Controls.GridViewColumn.HeaderStringFormat property.
  """
  pass
 def OnPropertyChanged(self,*args):
  """
  OnPropertyChanged(self: GridViewColumn,e: PropertyChangedEventArgs)

   Raises the 

    System.Windows.Controls.GridViewColumn.System#ComponentModel#INotifyPropertyChanged#PropertyChang

    ed event.

  

  

   e: The event data.

  OnPropertyChanged(self: DependencyObject,e: DependencyPropertyChangedEventArgs)

   Invoked whenever the effective value of any dependency property on this 

    System.Windows.DependencyObject has been updated. The specific dependency property that changed 

    is reported in the event data.

  

  

   e: Event data that will contain the dependency property identifier of interest,the property 

    metadata for the type,and old and new values.
  """
  pass
 def ShouldSerializeProperty(self,*args):
  """
  ShouldSerializeProperty(self: DependencyObject,dp: DependencyProperty) -> bool

  

   Returns a value that indicates whether serialization processes should serialize the value for 

    the provided dependency property.

  

  

   dp: The identifier for the dependency property that should be serialized.

   Returns: true if the dependency property that is supplied should be value-serialized; otherwise,false.
  """
  pass
 def ToString(self):
  """
  ToString(self: GridViewColumn) -> str

  

   Creates a string representation of the System.Windows.Controls.GridViewColumn.

   Returns: A string that identifies the object as a System.Windows.Controls.GridViewColumn object and 

    displays the value of the System.Windows.Controls.GridViewColumn.Header property.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass
 ActualWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the actual width of a System.Windows.Controls.GridViewColumn.



Get: ActualWidth(self: GridViewColumn) -> float



"""

 CellTemplate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the template to use to display the contents of a column cell.



Get: CellTemplate(self: GridViewColumn) -> DataTemplate



Set: CellTemplate(self: GridViewColumn)=value

"""

 CellTemplateSelector=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a System.Windows.Controls.DataTemplateSelector that determines the template to use to display cells in a column.



Get: CellTemplateSelector(self: GridViewColumn) -> DataTemplateSelector



Set: CellTemplateSelector(self: GridViewColumn)=value

"""

 DisplayMemberBinding=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the data item to bind to for this column.



Get: DisplayMemberBinding(self: GridViewColumn) -> BindingBase



Set: DisplayMemberBinding(self: GridViewColumn)=value

"""

 Header=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the content of the header of a System.Windows.Controls.GridViewColumn.



Get: Header(self: GridViewColumn) -> object



Set: Header(self: GridViewColumn)=value

"""

 HeaderContainerStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the style to use for the header of the System.Windows.Controls.GridViewColumn.



Get: HeaderContainerStyle(self: GridViewColumn) -> Style



Set: HeaderContainerStyle(self: GridViewColumn)=value

"""

 HeaderStringFormat=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a composite string that specifies how to format the System.Windows.Controls.GridViewColumn.Header property if it is displayed as a string.



Get: HeaderStringFormat(self: GridViewColumn) -> str



Set: HeaderStringFormat(self: GridViewColumn)=value

"""

 HeaderTemplate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the template to use to display the content of the column header.



Get: HeaderTemplate(self: GridViewColumn) -> DataTemplate



Set: HeaderTemplate(self: GridViewColumn)=value

"""

 HeaderTemplateSelector=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Controls.DataTemplateSelector that provides logic to select the template to use to display the column header.



Get: HeaderTemplateSelector(self: GridViewColumn) -> DataTemplateSelector



Set: HeaderTemplateSelector(self: GridViewColumn)=value

"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the width of the column.



Get: Width(self: GridViewColumn) -> float



Set: Width(self: GridViewColumn)=value

"""


 CellTemplateProperty=None
 CellTemplateSelectorProperty=None
 HeaderContainerStyleProperty=None
 HeaderProperty=None
 HeaderStringFormatProperty=None
 HeaderTemplateProperty=None
 HeaderTemplateSelectorProperty=None
 WidthProperty=None




class ToolStripContentPanelRenderEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.ToolStripContentPanel.RendererChanged event.

 

 ToolStripContentPanelRenderEventArgs(g: Graphics,contentPanel: ToolStripContentPanel)
 """
 @staticmethod
 def __new__(self,g,contentPanel):
  """ __new__(cls: type,g: Graphics,contentPanel: ToolStripContentPanel) """
  pass
 Graphics=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the object to use for drawing.



Get: Graphics(self: ToolStripContentPanelRenderEventArgs) -> Graphics



"""

 Handled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the event was handled.



Get: Handled(self: ToolStripContentPanelRenderEventArgs) -> bool



Set: Handled(self: ToolStripContentPanelRenderEventArgs)=value

"""

 ToolStripContentPanel=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.ToolStripContentPanel affected by the click.



Get: ToolStripContentPanel(self: ToolStripContentPanelRenderEventArgs) -> ToolStripContentPanel



"""





class Message(object):
 """ Implements a Windows message. """
 @staticmethod
 def Create(hWnd,msg,wparam,lparam):
  """
  Create(hWnd: IntPtr,msg: int,wparam: IntPtr,lparam: IntPtr) -> Message

  

   Creates a new System.Windows.Forms.Message.

  

   hWnd: The window handle that the message is for.

   msg: The message ID.

   wparam: The message wparam field.

   lparam: The message lparam field.

   Returns: A System.Windows.Forms.Message that represents the message that was created.
  """
  pass
 def Equals(self,o):
  """
  Equals(self: Message,o: object) -> bool

  

   Determines whether the specified object is equal to the current object.

  

   o: The object to compare with the current object.

   Returns: true if the specified object is equal to the current object; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: Message) -> int

   Returns: A 32-bit signed integer that is the hash code for this instance.
  """
  pass
 def GetLParam(self,cls):
  """
  GetLParam(self: Message,cls: Type) -> object

  

   Gets the System.Windows.Forms.Message.LParam value and converts the value to an object.

  

   cls: The type to use to create an instance. This type must be declared as a structure type.

   Returns: An System.Object that represents an instance of the class specified by the cls parameter,with 

    the data from the System.Windows.Forms.Message.LParam field of the message.
  """
  pass
 def ToString(self):
  """
  ToString(self: Message) -> str

  

   Returns a System.String that represents the current System.Windows.Forms.Message.

   Returns: A System.String that represents the current System.Windows.Forms.Message.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __ne__(self,*args):
  pass
 HWnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the window handle of the message.



Get: HWnd(self: Message) -> IntPtr



Set: HWnd(self: Message)=value

"""

 LParam=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Specifies the System.Windows.Forms.Message.LParam field of the message.



Get: LParam(self: Message) -> IntPtr



Set: LParam(self: Message)=value

"""

 Msg=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the ID number for the message.



Get: Msg(self: Message) -> int



Set: Msg(self: Message)=value

"""

 Result=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Specifies the value that is returned to Windows in response to handling the message.



Get: Result(self: Message) -> IntPtr



Set: Result(self: Message)=value

"""

 WParam=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Forms.Message.WParam field of the message.



Get: WParam(self: Message) -> IntPtr



Set: WParam(self: Message)=value

"""





class HtmlElementCollection(object,ICollection,IEnumerable):
 """ Defines a collection of System.Windows.Forms.HtmlElement objects. """
 def GetElementsByName(self,name):
  """
  GetElementsByName(self: HtmlElementCollection,name: str) -> HtmlElementCollection

  

   Gets a collection of elements by their name.

  

   name: The name or ID of the element.

   Returns: An System.Windows.Forms.HtmlElementCollection containing the elements whose 

    System.Windows.Forms.HtmlElement.Name property match name.
  """
  pass
 def GetEnumerator(self):
  """
  GetEnumerator(self: HtmlElementCollection) -> IEnumerator

  

   Returns an enumerator that iterates through a collection.

   Returns: An System.Collections.IEnumerator that can be used to iterate through the collection.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of elements in the collection.



Get: Count(self: HtmlElementCollection) -> int



"""





from typing import Tuple, Set, Iterable, List


class PipeFlowConfigurationType:
    Calculated = 0
    Preset = 1
    System = 2
    Demand = 3


class PipeFlowState:
    LaminarState = 0
    TransitionState = 1
    TurbulentState = 2
    MultiValues = -1


class PipeSystemType:
    UndefinedSystemType = 0
    SupplyHydronic = 7
    ReturnHydronic = 8
    Sanitary = 16
    Vent = 17
    DomesticHotWater = 19
    DomesticColdWater = 20
    OtherPipe = 22
    FireProtectWet = 23
    FireProtectDry = 24
    FireProtectPreaction = 25
    FireProtectOther = 26
    Fitting = 28
    Global = 29


class PipeLossMethodType:
    NotDefined = 0
    Table = 1
    SpecificLoss = 4
    Coefficient = 6


class FlexPipeType(MEPCurveType):


class FlexPipe(MEPCurve):
    @property
    def FlowState(self) -> PipeFlowState: ...
    @property
    def FlexPipeType(self) -> FlexPipeType: ...
    @FlexPipeType.setter
    def FlexPipeType(self, flexPipeType: FlexPipeType) -> None: ...
    @property
    def Points(self) -> List[XYZ]: ...
    @Points.setter
    def Points(self, points: List[XYZ]) -> None: ...
    @property
    def StartTangent(self) -> XYZ: ...
    @StartTangent.setter
    def StartTangent(self, tangent: XYZ) -> None: ...
    @property
    def EndTangent(self) -> XYZ: ...
    @EndTangent.setter
    def EndTangent(self, tangent: XYZ) -> None: ...
    @overload
    def Create(document: Document, systemTypeId: ElementId, pipeTypeId: ElementId, levelId: ElementId, startTangent: XYZ, endTangent: XYZ, points: List[XYZ]) -> FlexPipe: ...
    @overload
    def Create(document: Document, systemTypeId: ElementId, pipeTypeId: ElementId, levelId: ElementId, points: List[XYZ]) -> FlexPipe: ...
    def IsFlexPipeTypeId(document: Document, pipeTypeId: ElementId) -> bool: ...
    def IsPipingSystemTypeId(document: Document, systemTypeId: ElementId) -> bool: ...


class Pipe(MEPCurve):
    @property
    def FlowState(self) -> PipeFlowState: ...
    @property
    def PipeType(self) -> PipeType: ...
    @PipeType.setter
    def PipeType(self, pipeType: PipeType) -> None: ...
    @property
    def IsPlaceholder(self) -> bool: ...
    @property
    def PipeSegment(self) -> PipeSegment: ...
    @overload
    def Create(document: Document, pipeTypeId: ElementId, levelId: ElementId, startConnector: Connector, endConnector: Connector) -> Pipe: ...
    @overload
    def Create(document: Document, pipeTypeId: ElementId, levelId: ElementId, startConnector: Connector, endPoint: XYZ) -> Pipe: ...
    @overload
    def Create(document: Document, systemTypeId: ElementId, pipeTypeId: ElementId, levelId: ElementId, startPoint: XYZ, endPoint: XYZ) -> Pipe: ...
    def CreatePlaceholder(document: Document, systemTypeId: ElementId, pipeTypeId: ElementId, levelId: ElementId, startPoint: XYZ, endPoint: XYZ) -> Pipe: ...
    def SetSystemType(self, systemTypeId: ElementId) -> None: ...
    def IsPipeTypeId(document: Document, pipeTypeId: ElementId) -> bool: ...
    def IsPipingConnector(connector: Connector) -> bool: ...
    def IsPipingSystemTypeId(document: Document, systemTypeId: ElementId) -> bool: ...


class PipeType(MEPCurveType):


class PipingSystem(MEPSystem):
    @property
    def SystemType(self) -> PipeSystemType: ...
    @property
    def BaseEquipmentConnector(self) -> Connector: ...
    @BaseEquipmentConnector.setter
    def BaseEquipmentConnector(self, baseEquipmentConnector: Connector) -> None: ...
    @property
    def PipingNetwork(self) -> ElementSet: ...
    @property
    def IsWellConnected(self) -> bool: ...
    def IsFlowServerMissing(self) -> bool: ...
    def IsPressureDropServerMissing(self) -> bool: ...
    @overload
    def Create(ADocument: Document, typeId: ElementId, name: str) -> PipingSystem: ...
    @overload
    def Create(ADocument: Document, typeId: ElementId) -> PipingSystem: ...
    def GetVolume(self) -> float: ...
    def GetFlow(self) -> float: ...
    def GetStaticPressure(self) -> float: ...
    def GetFixtureUnits(self) -> float: ...
    def GetPumpSets(self) -> ISet: ...
    def CreateHydraulicSeparation(document: Document, pipeElementIds: ISet) -> ISet: ...
    def DeleteHydraulicSeparation(document: Document, pipeElementIds: ISet) -> None: ...
    def IsHydraulicLoopBoundary(element: Element) -> bool: ...
    def CanBeHydraulicLoopBoundary(element: Element) -> bool: ...


class IPipeFittingAndAccessoryPressureDropServer:
    def Calculate(self, data: PipeFittingAndAccessoryPressureDropData) -> bool: ...
    def IsApplicable(self, data: PipeFittingAndAccessoryPressureDropData) -> bool: ...
    def GetDataSchema(self) -> Schema: ...


class PipeFittingAndAccessoryPressureDropItem:
    @property
    def BeginConnectorIndex(self) -> int: ...
    @property
    def EndConnectorIndex(self) -> int: ...
    @property
    def VelocityPressure(self) -> float: ...
    @property
    def Coefficient(self) -> float: ...
    @Coefficient.setter
    def Coefficient(self, coefficient: float) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class PipeFittingAndAccessoryPressureDropData:
    @property
    def CalculationType(self) -> int: ...
    @property
    def IsCurrentEntityValid(self) -> bool: ...
    @IsCurrentEntityValid.setter
    def IsCurrentEntityValid(self, isCurrentEntityValid: bool) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetPipeFittingAndAccessoryData(self) -> PipeFittingAndAccessoryData: ...
    def GetPresureDropItems(self) -> List[PipeFittingAndAccessoryPressureDropItem]: ...
    def SetDefaultEntity(self, defaultEntity: Entity) -> None: ...
    def Dispose(self) -> None: ...


class IPipePlumbingFixtureFlowServer:
    def Calculate(self, data: PipePlumbingFixtureFlowData) -> None: ...
    def GetInformationLink(self) -> str: ...
    def GetHtmlDescription(self) -> str: ...


class PipingSystemType(MEPSystemType):
    @property
    def FlowConversionMethod(self) -> FlowConversionMode: ...
    @FlowConversionMethod.setter
    def FlowConversionMethod(self, flowConversionMethod: FlowConversionMode) -> None: ...
    @property
    def TwoLineRiseType(self) -> RiseDropSymbol: ...
    @TwoLineRiseType.setter
    def TwoLineRiseType(self, TwoLineRiseType: RiseDropSymbol) -> None: ...
    @property
    def TwoLineDropType(self) -> RiseDropSymbol: ...
    @TwoLineDropType.setter
    def TwoLineDropType(self, TwoLineDropType: RiseDropSymbol) -> None: ...
    @property
    def SingleLineBendRiseType(self) -> RiseDropSymbol: ...
    @SingleLineBendRiseType.setter
    def SingleLineBendRiseType(self, SingleLineBendRiseType: RiseDropSymbol) -> None: ...
    @property
    def SingleLineBendDropType(self) -> RiseDropSymbol: ...
    @SingleLineBendDropType.setter
    def SingleLineBendDropType(self, SingleLineBendDropType: RiseDropSymbol) -> None: ...
    @property
    def SingleLineJunctionRiseType(self) -> RiseDropSymbol: ...
    @SingleLineJunctionRiseType.setter
    def SingleLineJunctionRiseType(self, SingleLineJunctionRiseType: RiseDropSymbol) -> None: ...
    @property
    def SingleLineJunctionDropType(self) -> RiseDropSymbol: ...
    @SingleLineJunctionDropType.setter
    def SingleLineJunctionDropType(self, SingleLineJunctionDropType: RiseDropSymbol) -> None: ...
    @property
    def FluidTemperature(self) -> float: ...
    @FluidTemperature.setter
    def FluidTemperature(self, fluidTemperature: float) -> None: ...
    @property
    def FluidType(self) -> ElementId: ...
    @FluidType.setter
    def FluidType(self, fluidTypeId: ElementId) -> None: ...
    def Create(ADoc: Document, systemClassification: MEPSystemClassification, name: str) -> PipingSystemType: ...
    def ValidateRiseDropSymbolType(self, risedropType: RiseDropSymbol) -> bool: ...


class PipePlumbingFixtureFlowData:
    @property
    def Flow(self) -> float: ...
    @Flow.setter
    def Flow(self, flow: float) -> None: ...
    @property
    def FlowConfiguration(self) -> PipeFlowConfigurationType: ...
    @property
    def FixtureUnits(self) -> float: ...
    @property
    def FlowConversionMode(self) -> FlowConversionMode: ...
    @property
    def DimensionFlow(self) -> float: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class IPipePressureDropServer:
    def Calculate(self, data: PipePressureDropData) -> None: ...
    def GetInformationLink(self) -> str: ...
    def GetHtmlDescription(self) -> str: ...


class PipePressureDropData:
    @property
    def RelativeRoughness(self) -> float: ...
    @RelativeRoughness.setter
    def RelativeRoughness(self, relativeRoughness: float) -> None: ...
    @property
    def ReynoldsNumber(self) -> float: ...
    @ReynoldsNumber.setter
    def ReynoldsNumber(self, reynoldsNumber: float) -> None: ...
    @property
    def FlowState(self) -> PipeFlowState: ...
    @FlowState.setter
    def FlowState(self, flowState: PipeFlowState) -> None: ...
    @property
    def Friction(self) -> float: ...
    @Friction.setter
    def Friction(self, friction: float) -> None: ...
    @property
    def FrictionFactor(self) -> float: ...
    @FrictionFactor.setter
    def FrictionFactor(self, frictionFactor: float) -> None: ...
    @property
    def Velocity(self) -> float: ...
    @Velocity.setter
    def Velocity(self, velocity: float) -> None: ...
    @property
    def VelocityPressure(self) -> float: ...
    @VelocityPressure.setter
    def VelocityPressure(self, velocityPressure: float) -> None: ...
    @property
    def Coefficient(self) -> float: ...
    @Coefficient.setter
    def Coefficient(self, coefficient: float) -> None: ...
    @property
    def PressureDrop(self) -> float: ...
    @PressureDrop.setter
    def PressureDrop(self, pressureDrop: float) -> None: ...
    @property
    def Roughness(self) -> float: ...
    @property
    def Length(self) -> float: ...
    @property
    def Flow(self) -> float: ...
    @property
    def InsideDiameter(self) -> float: ...
    @property
    def OutsideDiameter(self) -> float: ...
    @property
    def NominalDiameter(self) -> float: ...
    @property
    def Density(self) -> float: ...
    @property
    def Viscosity(self) -> float: ...
    @property
    def CategoryId(self) -> ElementId: ...
    @property
    def KLevel(self) -> SystemCalculationLevel: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class FlowConversionMode:
    Valves = 0
    Tanks = 1
    Invalid = -1


class PipeFittingAndAccessoryConnectorData:
    @property
    def Width(self) -> float: ...
    @property
    def Height(self) -> float: ...
    @property
    def Diameter(self) -> float: ...
    @property
    def Angle(self) -> float: ...
    @property
    def Index(self) -> int: ...
    @property
    def LinkIndex(self) -> int: ...
    @property
    def FlowDirection(self) -> FlowDirectionType: ...
    @property
    def Flow(self) -> float: ...
    @property
    def VelocityPressure(self) -> float: ...
    @property
    def Profile(self) -> ConnectorProfileType: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetCoordination(self) -> Transform: ...
    def Dispose(self) -> None: ...


class PipeFittingAndAccessoryData:
    @property
    def ServerGUID(self) -> Guid: ...
    @property
    def PartType(self) -> PartType: ...
    @property
    def BehaviorType(self) -> int: ...
    @property
    def SystemClassification(self) -> MEPSystemClassification: ...
    @property
    def Origin(self) -> XYZ: ...
    @property
    def FluidViscosity(self) -> float: ...
    @property
    def FluidDensity(self) -> float: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetEntity(self) -> Entity: ...
    def GetAllConnectorData(self) -> List[PipeFittingAndAccessoryConnectorData]: ...
    def GetFamilyInstanceId(self) -> ElementId: ...
    def Dispose(self) -> None: ...


class PipeSegment(Segment):
    @property
    def ScheduleTypeId(self) -> ElementId: ...
    def Create(ADocument: Document, MaterialId: ElementId, ScheduleId: ElementId, sizeSet: ICollection) -> PipeSegment: ...


class PlumbingUtils:
    def ConvertPipePlaceholders(document: Document, placeholderIds: ICollection) -> ICollection: ...
    @overload
    def ConnectPipePlaceholdersAtElbow(document: Document, connector1: Connector, connector2: Connector) -> bool: ...
    @overload
    def ConnectPipePlaceholdersAtElbow(document: Document, placeholder1Id: ElementId, placeholder2Id: ElementId) -> bool: ...
    @overload
    def ConnectPipePlaceholdersAtTee(document: Document, connector1: Connector, connector2: Connector, connector3: Connector) -> bool: ...
    @overload
    def ConnectPipePlaceholdersAtTee(document: Document, placeholder1Id: ElementId, placeholder2Id: ElementId) -> bool: ...
    @overload
    def ConnectPipePlaceholdersAtCross(document: Document, connector1: Connector, connector2: Connector, connector3: Connector, connector4: Connector) -> bool: ...
    @overload
    def ConnectPipePlaceholdersAtCross(document: Document, placeholder1Id: ElementId, placeholder2Id: ElementId, placeholder3Id: ElementId) -> bool: ...
    @overload
    def ConnectPipePlaceholdersAtCross(document: Document, placeholder1Id: ElementId, placeholder2Id: ElementId) -> bool: ...
    def PlaceCapOnOpenEnds(document: Document, elemId: ElementId, typeId: ElementId) -> None: ...
    def HasOpenConnector(document: Document, elemId: ElementId) -> bool: ...
    def BreakCurve(document: Document, pipeId: ElementId, ptBreak: XYZ) -> ElementId: ...


class FluidTemperatureSetIterator:
    @property
    def IsValidObject(self) -> bool: ...
    def MoveNext(self) -> bool: ...
    def IsDone(self) -> bool: ...
    def Reset(self) -> None: ...
    def GetCurrent(self) -> FluidTemperature: ...
    @property
    def Current(self) -> FluidTemperature: ...
    def Dispose(self) -> None: ...


class FluidTemperature:
    def __init__(self, temperature: float, viscosity: float, density: float): ...
    @property
    def Temperature(self) -> float: ...
    @Temperature.setter
    def Temperature(self, temperature: float) -> None: ...
    @property
    def Viscosity(self) -> float: ...
    @Viscosity.setter
    def Viscosity(self, viscosity: float) -> None: ...
    @property
    def Density(self) -> float: ...
    @Density.setter
    def Density(self, density: float) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class FluidType(ElementType):
    def GetFluidTemperatureSetIterator(self) -> FluidTemperatureSetIterator: ...
    @overload
    def Create(document: Document, fluidTypeName: str, basedOnFluidType: FluidType) -> FluidType: ...
    @overload
    def Create(document: Document, fluidTypeName: str) -> FluidType: ...
    def GetFluidType(document: Document, fluidTypeName: str) -> FluidType: ...
    def IsFluidInUse(document: Document, fluidId: ElementId) -> bool: ...
    def ClearAllTemperatures(self) -> None: ...
    def GetTemperature(self, temperature: float) -> FluidTemperature: ...
    def AddTemperature(self, fluidTemperature: FluidTemperature) -> None: ...
    def RemoveTemperature(self, temperature: float) -> None: ...
    def GetEnumerator(self) -> IEnumerator: ...


class PipeInsulation(InsulationLiningBase):
    def Create(document: Document, pipeOrContentElementId: ElementId, pipeInsulationTypeId: ElementId, Thickness: float) -> PipeInsulation: ...


class PipeInsulationType(ElementType):


class PipeScheduleType(ElementType):
    def Create(doc: Document, name: str) -> PipeScheduleType: ...
    def GetPipeScheduleId(doc: Document, name: str) -> ElementId: ...


class PipeSettings(Element):
    @property
    def FlatOnTop(self) -> str: ...
    @FlatOnTop.setter
    def FlatOnTop(self, flatOnTop: str) -> None: ...
    @property
    def FlatOnBottom(self) -> str: ...
    @FlatOnBottom.setter
    def FlatOnBottom(self, flatOnBottom: str) -> None: ...
    @property
    def SetUp(self) -> str: ...
    @SetUp.setter
    def SetUp(self, setUp: str) -> None: ...
    @property
    def SetDown(self) -> str: ...
    @SetDown.setter
    def SetDown(self, setDown: str) -> None: ...
    @property
    def SetUpFromBottom(self) -> str: ...
    @SetUpFromBottom.setter
    def SetUpFromBottom(self, setUpFromBottom: str) -> None: ...
    @property
    def SetDownFromBottom(self) -> str: ...
    @SetDownFromBottom.setter
    def SetDownFromBottom(self, setDownFromBottom: str) -> None: ...
    @property
    def Centerline(self) -> str: ...
    @Centerline.setter
    def Centerline(self, centerline: str) -> None: ...
    @property
    def FittingAngleUsage(self) -> FittingAngleUsage: ...
    @FittingAngleUsage.setter
    def FittingAngleUsage(self, fittingAngleUsage: FittingAngleUsage) -> None: ...
    @property
    def SizeSuffix(self) -> str: ...
    @SizeSuffix.setter
    def SizeSuffix(self, str: str) -> None: ...
    @property
    def SizePrefix(self) -> str: ...
    @SizePrefix.setter
    def SizePrefix(self, str: str) -> None: ...
    @property
    def ConnectorSeparator(self) -> str: ...
    @ConnectorSeparator.setter
    def ConnectorSeparator(self, str: str) -> None: ...
    @property
    def ConnectorTolerance(self) -> float: ...
    @ConnectorTolerance.setter
    def ConnectorTolerance(self, dValue: float) -> None: ...
    @property
    def FittingAnnotationSize(self) -> float: ...
    @FittingAnnotationSize.setter
    def FittingAnnotationSize(self, dValue: float) -> None: ...
    @property
    def UseAnnotationScaleForSingleLineFittings(self) -> bool: ...
    @UseAnnotationScaleForSingleLineFittings.setter
    def UseAnnotationScaleForSingleLineFittings(self, value: bool) -> None: ...
    @property
    def AnalysisForClosedLoopHydronicPipingNetworks(self) -> bool: ...
    @AnalysisForClosedLoopHydronicPipingNetworks.setter
    def AnalysisForClosedLoopHydronicPipingNetworks(self, value: bool) -> None: ...
    def GetPipeSettings(document: Document) -> PipeSettings: ...
    def GetPipeSlopes(self) -> List[float]: ...
    def SetPipeSlopes(self, slopes: List[float]) -> None: ...
    def AddPipeSlope(self, slope: float) -> None: ...
    def GetSpecificFittingAngles(self) -> List[float]: ...
    def SetSpecificFittingAngleStatus(self, angle: float, bStatus: bool) -> None: ...
    def GetSpecificFittingAngleStatus(self, angle: float) -> bool: ...
    def IsAnalysisForClosedLoopHydronicPipingNetworksEnabled(ccda: Document) -> bool: ...
    def IsValidSpecificFittingAngle(self, angle: float) -> bool: ...
    def GetFlowConvertionServerInfo(self) -> MEPCalculationServerInfo: ...
    def SetFlowConvertionServerInfo(self, serverInfo: MEPCalculationServerInfo) -> None: ...



# -*- coding: utf-8 -*-
"""HTTP API framework similar to flask."""
# types to be exported
#pylint: disable=wildcard-import,redefined-outer-name,unused-variable
#pylint: disable=dangerous-default-value,invalid-name
from pyrevit.routes.server import *


class API(object):
    """API root object.

    Args:
        name (str): URL-safe unique root name of the API

    Examples:
        ```python
        from pyrevit import routes
        api = routes.API("pyrevit-core")
        @api.route('/sessions/', methods=['POST'])
        def reload_pyrevit(uiapp):
            new_session_id = sessionmgr.reload_pyrevit()
            return {"session_id": new_session_id}
        ```
    """
    def __init__(self, name):
        self.name = name

    def route(self, pattern, methods=['GET']):
        """Define a new route on this API."""
        def __func_wrapper__(f):
            for method in methods:
                add_route(
                    api_name=self.name,
                    pattern=pattern,
                    method=method,
                    handler_func=f
                    )
            return f
        return __func_wrapper__



def active_routes_api():
    """Activates routes API."""
    from pyrevit.routes import api



class Attribute(object,_Attribute):
 """ Represents the base class for custom attributes. """
 def Equals(self,obj):
  """
  Equals(self: Attribute,obj: object) -> bool

  

   Returns a value that indicates whether this instance is equal to a specified object.

  

   obj: An System.Object to compare with this instance or null.

   Returns: true if obj equals the type and value of this instance; otherwise,false.
  """
  pass
 @staticmethod
 def GetCustomAttribute(element,attributeType,inherit=None):
  """
  GetCustomAttribute(element: Module,attributeType: Type,inherit: bool) -> Attribute

  

   Retrieves a custom attribute applied to a module. Parameters specify the module,the type of the 

    custom attribute to search for,and an ignored search option.

  

  

   element: An object derived from the System.Reflection.Module class that describes a portable executable 

    file.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: This parameter is ignored,and does not affect the operation of this method.

   Returns: A reference to the single custom attribute of type attributeType that is applied to element,or 

    null if there is no such attribute.

  

  GetCustomAttribute(element: Module,attributeType: Type) -> Attribute

  

   Retrieves a custom attribute applied to a module. Parameters specify the module,and the type of 

    the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.Module class that describes a portable executable 

    file.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: A reference to the single custom attribute of type attributeType that is applied to element,or 

    null if there is no such attribute.

  

  GetCustomAttribute(element: Assembly,attributeType: Type,inherit: bool) -> Attribute

  

   Retrieves a custom attribute applied to an assembly. Parameters specify the assembly,the type 

    of the custom attribute to search for,and an ignored search option.

  

  

   element: An object derived from the System.Reflection.Assembly class that describes a reusable collection 

    of modules.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: This parameter is ignored,and does not affect the operation of this method.

   Returns: A reference to the single custom attribute of type attributeType that is applied to element,or 

    null if there is no such attribute.

  

  GetCustomAttribute(element: Assembly,attributeType: Type) -> Attribute

  

   Retrieves a custom attribute applied to a specified assembly. Parameters specify the assembly 

    and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.Assembly class that describes a reusable collection 

    of modules.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: A reference to the single custom attribute of type attributeType that is applied to element,or 

    null if there is no such attribute.

  

  GetCustomAttribute(element: MemberInfo,attributeType: Type,inherit: bool) -> Attribute

  

   Retrieves a custom attribute applied to a member of a type. Parameters specify the member,the 

    type of the custom attribute to search for,and whether to search ancestors of the member.

  

  

   element: An object derived from the System.Reflection.MemberInfo class that describes a constructor,

    event,field,method,or property member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: If true,specifies to also search the ancestors of element for custom attributes.

   Returns: A reference to the single custom attribute of type attributeType that is applied to element,or 

    null if there is no such attribute.

  

  GetCustomAttribute(element: MemberInfo,attributeType: Type) -> Attribute

  

   Retrieves a custom attribute applied to a member of a type. Parameters specify the member,and 

    the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.MemberInfo class that describes a constructor,

    event,field,method,or property member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: A reference to the single custom attribute of type attributeType that is applied to element,or 

    null if there is no such attribute.

  

  GetCustomAttribute(element: ParameterInfo,attributeType: Type,inherit: bool) -> Attribute

  

   Retrieves a custom attribute applied to a method parameter. Parameters specify the method 

    parameter,the type of the custom attribute to search for,and whether to search ancestors of 

    the method parameter.

  

  

   element: An object derived from the System.Reflection.ParameterInfo class that describes a parameter of a 

    member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: If true,specifies to also search the ancestors of element for custom attributes.

   Returns: A reference to the single custom attribute of type attributeType that is applied to element,or 

    null if there is no such attribute.

  

  GetCustomAttribute(element: ParameterInfo,attributeType: Type) -> Attribute

  

   Retrieves a custom attribute applied to a method parameter. Parameters specify the method 

    parameter,and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.ParameterInfo class that describes a parameter of a 

    member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: A reference to the single custom attribute of type attributeType that is applied to element,or 

    null if there is no such attribute.
  """
  pass
 @staticmethod
 def GetCustomAttributes(element,*__args):
  """
  GetCustomAttributes(element: Module,inherit: bool) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a module. Parameters specify the module,

    and an ignored search option.

  

  

   element: An object derived from the System.Reflection.Module class that describes a portable executable 

    file.

  

   inherit: This parameter is ignored,and does not affect the operation of this method.

   Returns: An System.Attribute array that contains the custom attributes applied to element,or an empty 

    array if no such custom attributes exist.

  

  GetCustomAttributes(element: Module,attributeType: Type,inherit: bool) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a module. Parameters specify the module,

    the type of the custom attribute to search for,and an ignored search option.

  

  

   element: An object derived from the System.Reflection.Module class that describes a portable executable 

    file.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: This parameter is ignored,and does not affect the operation of this method.

   Returns: An System.Attribute array that contains the custom attributes of type attributeType applied to 

    element,or an empty array if no such custom attributes exist.

  

  GetCustomAttributes(element: Module,attributeType: Type) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a module. Parameters specify the module,

    and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.Module class that describes a portable executable 

    file.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: An System.Attribute array that contains the custom attributes of type attributeType applied to 

    element,or an empty array if no such custom attributes exist.

  

  GetCustomAttributes(element: Module) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a module. A parameter specifies the 

    module.

  

  

   element: An object derived from the System.Reflection.Module class that describes a portable executable 

    file.

  

   Returns: An System.Attribute array that contains the custom attributes applied to element,or an empty 

    array if no such custom attributes exist.

  

  GetCustomAttributes(element: Assembly) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to an assembly. A parameter specifies the 

    assembly.

  

  

   element: An object derived from the System.Reflection.Assembly class that describes a reusable collection 

    of modules.

  

   Returns: An System.Attribute array that contains the custom attributes applied to element,or an empty 

    array if no such custom attributes exist.

  

  GetCustomAttributes(element: Assembly,inherit: bool) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to an assembly. Parameters specify the 

    assembly,and an ignored search option.

  

  

   element: An object derived from the System.Reflection.Assembly class that describes a reusable collection 

    of modules.

  

   inherit: This parameter is ignored,and does not affect the operation of this method.

   Returns: An System.Attribute array that contains the custom attributes applied to element,or an empty 

    array if no such custom attributes exist.

  

  GetCustomAttributes(element: Assembly,attributeType: Type) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to an assembly. Parameters specify the 

    assembly,and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.Assembly class that describes a reusable collection 

    of modules.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: An System.Attribute array that contains the custom attributes of type attributeType applied to 

    element,or an empty array if no such custom attributes exist.

  

  GetCustomAttributes(element: Assembly,attributeType: Type,inherit: bool) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to an assembly. Parameters specify the 

    assembly,the type of the custom attribute to search for,and an ignored search option.

  

  

   element: An object derived from the System.Reflection.Assembly class that describes a reusable collection 

    of modules.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: This parameter is ignored,and does not affect the operation of this method.

   Returns: An System.Attribute array that contains the custom attributes of type attributeType applied to 

    element,or an empty array if no such custom attributes exist.

  

  GetCustomAttributes(element: MemberInfo) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a member of a type. A parameter specifies 

    the member.

  

  

   element: An object derived from the System.Reflection.MemberInfo class that describes a constructor,

    event,field,method,or property member of a class.

  

   Returns: An System.Attribute array that contains the custom attributes applied to element,or an empty 

    array if no such custom attributes exist.

  

  GetCustomAttributes(element: MemberInfo,inherit: bool) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a member of a type. Parameters specify 

    the member,the type of the custom attribute to search for,and whether to search ancestors of 

    the member.

  

  

   element: An object derived from the System.Reflection.MemberInfo class that describes a constructor,

    event,field,method,or property member of a class.

  

   inherit: If true,specifies to also search the ancestors of element for custom attributes.

   Returns: An System.Attribute array that contains the custom attributes applied to element,or an empty 

    array if no such custom attributes exist.

  

  GetCustomAttributes(element: MemberInfo,type: Type) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a member of a type. Parameters specify 

    the member,and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.MemberInfo class that describes a constructor,

    event,field,method,or property member of a class.

  

   type: The type,or a base type,of the custom attribute to search for.

   Returns: An System.Attribute array that contains the custom attributes of type type applied to element,

    or an empty array if no such custom attributes exist.

  

  GetCustomAttributes(element: MemberInfo,type: Type,inherit: bool) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a member of a type. Parameters specify 

    the member,the type of the custom attribute to search for,and whether to search ancestors of 

    the member.

  

  

   element: An object derived from the System.Reflection.MemberInfo class that describes a constructor,

    event,field,method,or property member of a class.

  

   type: The type,or a base type,of the custom attribute to search for.

   inherit: If true,specifies to also search the ancestors of element for custom attributes.

   Returns: An System.Attribute array that contains the custom attributes of type type applied to element,

    or an empty array if no such custom attributes exist.

  

  GetCustomAttributes(element: ParameterInfo,attributeType: Type,inherit: bool) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a method parameter. Parameters specify 

    the method parameter,the type of the custom attribute to search for,and whether to search 

    ancestors of the method parameter.

  

  

   element: An object derived from the System.Reflection.ParameterInfo class that describes a parameter of a 

    member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: If true,specifies to also search the ancestors of element for custom attributes.

   Returns: An System.Attribute array that contains the custom attributes of type attributeType applied to 

    element,or an empty array if no such custom attributes exist.

  

  GetCustomAttributes(element: ParameterInfo,inherit: bool) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a method parameter. Parameters specify 

    the method parameter,and whether to search ancestors of the method parameter.

  

  

   element: An object derived from the System.Reflection.ParameterInfo class that describes a parameter of a 

    member of a class.

  

   inherit: If true,specifies to also search the ancestors of element for custom attributes.

   Returns: An System.Attribute array that contains the custom attributes applied to element,or an empty 

    array if no such custom attributes exist.

  

  GetCustomAttributes(element: ParameterInfo) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a method parameter. A parameter specifies 

    the method parameter.

  

  

   element: An object derived from the System.Reflection.ParameterInfo class that describes a parameter of a 

    member of a class.

  

   Returns: An System.Attribute array that contains the custom attributes applied to element,or an empty 

    array if no such custom attributes exist.

  

  GetCustomAttributes(element: ParameterInfo,attributeType: Type) -> Array[Attribute]

  

   Retrieves an array of the custom attributes applied to a method parameter. Parameters specify 

    the method parameter,and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.ParameterInfo class that describes a parameter of a 

    member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: An System.Attribute array that contains the custom attributes of type attributeType applied to 

    element,or an empty array if no such custom attributes exist.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: Attribute) -> int

  

   Returns the hash code for this instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def IsDefaultAttribute(self):
  """
  IsDefaultAttribute(self: Attribute) -> bool

  

   When overridden in a derived class,indicates whether the value of this instance is the default 

    value for the derived class.

  

   Returns: true if this instance is the default attribute for the class; otherwise,false.
  """
  pass
 @staticmethod
 def IsDefined(element,attributeType,inherit=None):
  """
  IsDefined(element: Module,attributeType: Type,inherit: bool) -> bool

  

   Determines whether any custom attributes are applied to a module. Parameters specify the module,

    the type of the custom attribute to search for,and an ignored search option.

  

  

   element: An object derived from the System.Reflection.Module class that describes a portable executable 

    file.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: This parameter is ignored,and does not affect the operation of this method.

   Returns: true if a custom attribute of type attributeType is applied to element; otherwise,false.

  IsDefined(element: Module,attributeType: Type) -> bool

  

   Determines whether any custom attributes of a specified type are applied to a module. Parameters 

    specify the module,and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.Module class that describes a portable executable 

    file.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: true if a custom attribute of type attributeType is applied to element; otherwise,false.

  IsDefined(element: Assembly,attributeType: Type,inherit: bool) -> bool

  

   Determines whether any custom attributes are applied to an assembly. Parameters specify the 

    assembly,the type of the custom attribute to search for,and an ignored search option.

  

  

   element: An object derived from the System.Reflection.Assembly class that describes a reusable collection 

    of modules.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: This parameter is ignored,and does not affect the operation of this method.

   Returns: true if a custom attribute of type attributeType is applied to element; otherwise,false.

  IsDefined(element: Assembly,attributeType: Type) -> bool

  

   Determines whether any custom attributes are applied to an assembly. Parameters specify the 

    assembly,and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.Assembly class that describes a reusable collection 

    of modules.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: true if a custom attribute of type attributeType is applied to element; otherwise,false.

  IsDefined(element: MemberInfo,attributeType: Type,inherit: bool) -> bool

  

   Determines whether any custom attributes are applied to a member of a type. Parameters specify 

    the member,the type of the custom attribute to search for,and whether to search ancestors of 

    the member.

  

  

   element: An object derived from the System.Reflection.MemberInfo class that describes a constructor,

    event,field,method,type,or property member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: If true,specifies to also search the ancestors of element for custom attributes.

   Returns: true if a custom attribute of type attributeType is applied to element; otherwise,false.

  IsDefined(element: MemberInfo,attributeType: Type) -> bool

  

   Determines whether any custom attributes are applied to a member of a type. Parameters specify 

    the member,and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.MemberInfo class that describes a constructor,

    event,field,method,type,or property member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: true if a custom attribute of type attributeType is applied to element; otherwise,false.

  IsDefined(element: ParameterInfo,attributeType: Type,inherit: bool) -> bool

  

   Determines whether any custom attributes are applied to a method parameter. Parameters specify 

    the method parameter,the type of the custom attribute to search for,and whether to search 

    ancestors of the method parameter.

  

  

   element: An object derived from the System.Reflection.ParameterInfo class that describes a parameter of a 

    member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   inherit: If true,specifies to also search the ancestors of element for custom attributes.

   Returns: true if a custom attribute of type attributeType is applied to element; otherwise,false.

  IsDefined(element: ParameterInfo,attributeType: Type) -> bool

  

   Determines whether any custom attributes are applied to a method parameter. Parameters specify 

    the method parameter,and the type of the custom attribute to search for.

  

  

   element: An object derived from the System.Reflection.ParameterInfo class that describes a parameter of a 

    member of a class.

  

   attributeType: The type,or a base type,of the custom attribute to search for.

   Returns: true if a custom attribute of type attributeType is applied to element; otherwise,false.
  """
  pass
 def Match(self,obj):
  """
  Match(self: Attribute,obj: object) -> bool

  

   When overridden in a derived class,returns a value that indicates whether this instance equals 

    a specified object.

  

  

   obj: An System.Object to compare with this instance of System.Attribute.

   Returns: true if this instance equals obj; otherwise,false.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 TypeId=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When implemented in a derived class,gets a unique identifier for this System.Attribute.



Get: TypeId(self: Attribute) -> object



"""





class DataGridViewCellPaintingEventArgs(HandledEventArgs):
 """
 Provides data for the System.Windows.Forms.DataGridView.CellPainting event.

 

 DataGridViewCellPaintingEventArgs(dataGridView: DataGridView,graphics: Graphics,clipBounds: Rectangle,cellBounds: Rectangle,rowIndex: int,columnIndex: int,cellState: DataGridViewElementStates,value: object,formattedValue: object,errorText: str,cellStyle: DataGridViewCellStyle,advancedBorderStyle: DataGridViewAdvancedBorderStyle,paintParts: DataGridViewPaintParts)
 """
 def Paint(self,clipBounds,paintParts):
  """
  Paint(self: DataGridViewCellPaintingEventArgs,clipBounds: Rectangle,paintParts: DataGridViewPaintParts)

   Paints the specified parts of the cell for the area in the specified bounds.

  

   clipBounds: A System.Drawing.Rectangle that specifies the area of the System.Windows.Forms.DataGridView to 

    be painted.

  

   paintParts: A bitwise combination of System.Windows.Forms.DataGridViewPaintParts values specifying the parts 

    to paint.
  """
  pass
 def PaintBackground(self,clipBounds,cellsPaintSelectionBackground):
  """
  PaintBackground(self: DataGridViewCellPaintingEventArgs,clipBounds: Rectangle,cellsPaintSelectionBackground: bool)

   Paints the cell background for the area in the specified bounds.

  

   clipBounds: A System.Drawing.Rectangle that specifies the area of the System.Windows.Forms.DataGridView to 

    be painted.

  

   cellsPaintSelectionBackground: true to paint the background of the specified bounds with the color of the 

    System.Windows.Forms.DataGridViewCellStyle.SelectionBackColor property of the 

    System.Windows.Forms.DataGridViewCell.InheritedStyle; false to paint the background of the 

    specified bounds with the color of the System.Windows.Forms.DataGridViewCellStyle.BackColor 

    property of the System.Windows.Forms.DataGridViewCell.InheritedStyle.
  """
  pass
 def PaintContent(self,clipBounds):
  """
  PaintContent(self: DataGridViewCellPaintingEventArgs,clipBounds: Rectangle)

   Paints the cell content for the area in the specified bounds.

  

   clipBounds: A System.Drawing.Rectangle that specifies the area of the System.Windows.Forms.DataGridView to 

    be painted.
  """
  pass
 @staticmethod
 def __new__(self,dataGridView,graphics,clipBounds,cellBounds,rowIndex,columnIndex,cellState,value,formattedValue,errorText,cellStyle,advancedBorderStyle,paintParts):
  """ __new__(cls: type,dataGridView: DataGridView,graphics: Graphics,clipBounds: Rectangle,cellBounds: Rectangle,rowIndex: int,columnIndex: int,cellState: DataGridViewElementStates,value: object,formattedValue: object,errorText: str,cellStyle: DataGridViewCellStyle,advancedBorderStyle: DataGridViewAdvancedBorderStyle,paintParts: DataGridViewPaintParts) """
  pass
 AdvancedBorderStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the border style of the current System.Windows.Forms.DataGridViewCell.



Get: AdvancedBorderStyle(self: DataGridViewCellPaintingEventArgs) -> DataGridViewAdvancedBorderStyle



"""

 CellBounds=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get the bounds of the current System.Windows.Forms.DataGridViewCell.



Get: CellBounds(self: DataGridViewCellPaintingEventArgs) -> Rectangle



"""

 CellStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the cell style of the current System.Windows.Forms.DataGridViewCell.



Get: CellStyle(self: DataGridViewCellPaintingEventArgs) -> DataGridViewCellStyle



"""

 ClipBounds=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the area of the System.Windows.Forms.DataGridView that needs to be repainted.



Get: ClipBounds(self: DataGridViewCellPaintingEventArgs) -> Rectangle



"""

 ColumnIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the column index of the current System.Windows.Forms.DataGridViewCell.



Get: ColumnIndex(self: DataGridViewCellPaintingEventArgs) -> int



"""

 ErrorText=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a string that represents an error message for the current System.Windows.Forms.DataGridViewCell.



Get: ErrorText(self: DataGridViewCellPaintingEventArgs) -> str



"""

 FormattedValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the formatted value of the current System.Windows.Forms.DataGridViewCell.



Get: FormattedValue(self: DataGridViewCellPaintingEventArgs) -> object



"""

 Graphics=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Drawing.Graphics used to paint the current System.Windows.Forms.DataGridViewCell.



Get: Graphics(self: DataGridViewCellPaintingEventArgs) -> Graphics



"""

 PaintParts=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """The cell parts that are to be painted.



Get: PaintParts(self: DataGridViewCellPaintingEventArgs) -> DataGridViewPaintParts



"""

 RowIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the row index of the current System.Windows.Forms.DataGridViewCell.



Get: RowIndex(self: DataGridViewCellPaintingEventArgs) -> int



"""

 State=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the state of the current System.Windows.Forms.DataGridViewCell.



Get: State(self: DataGridViewCellPaintingEventArgs) -> DataGridViewElementStates



"""

 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the value of the current System.Windows.Forms.DataGridViewCell.



Get: Value(self: DataGridViewCellPaintingEventArgs) -> object



"""





class GlobalProxySelection(object):
 """
 Contains a global default proxy instance for all HTTP requests.

 

 GlobalProxySelection()
 """
 @staticmethod
 def GetEmptyWebProxy():
  """
  GetEmptyWebProxy() -> IWebProxy

  

   Returns an empty proxy instance.

   Returns: An System.Net.IWebProxy that contains no information.
  """
  pass
 Select=None




class VScrollProperties(ScrollProperties):
 """
 Provides basic properties for the System.Windows.Forms.VScrollBar class.

 

 VScrollProperties(container: ScrollableControl)
 """
 @staticmethod
 def __new__(self,container):
  """ __new__(cls: type,container: ScrollableControl) """
  pass
 ParentControl=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the control to which this scroll information applies.



"""





class TableLayoutSettings(LayoutSettings,ISerializable):
 """ Collects the characteristics associated with table layouts. """
 def GetCellPosition(self,control):
  """
  GetCellPosition(self: TableLayoutSettings,control: object) -> TableLayoutPanelCellPosition

  

   Gets the System.Windows.Forms.TableLayoutPanelCellPosition that represents the row and the 

    column of the cell.

  

  

   control: A control contained within a cell.

   Returns: A System.Windows.Forms.TableLayoutPanelCellPosition that represents the cell position.
  """
  pass
 def GetColumn(self,control):
  """
  GetColumn(self: TableLayoutSettings,control: object) -> int

  

   Gets the column position of the specified child control.

  

   control: A control contained within a cell.

   Returns: The column position of the specified child control.
  """
  pass
 def GetColumnSpan(self,control):
  """
  GetColumnSpan(self: TableLayoutSettings,control: object) -> int

  

   Gets the number of columns that the cell containing the child control spans.

  

   control: A control contained within a cell.

   Returns: The number of columns that the cell containing the child control spans.
  """
  pass
 def GetRow(self,control):
  """
  GetRow(self: TableLayoutSettings,control: object) -> int

  

   Gets the row position of the specified child control.

  

   control: A control contained within a cell.

   Returns: The row position of the specified child control.
  """
  pass
 def GetRowSpan(self,control):
  """
  GetRowSpan(self: TableLayoutSettings,control: object) -> int

  

   Gets the number of rows that the cell containing the child control spans.

  

   control: A control contained within a cell.

   Returns: The number of rows that the cell containing the child control spans.
  """
  pass
 def SetCellPosition(self,control,cellPosition):
  """
  SetCellPosition(self: TableLayoutSettings,control: object,cellPosition: TableLayoutPanelCellPosition)

   Sets the System.Windows.Forms.TableLayoutPanelCellPosition that represents the row and the 

    column of the cell.

  

  

   control: A control contained within a cell.

   cellPosition: A System.Windows.Forms.TableLayoutPanelCellPosition  that represents the cell position.
  """
  pass
 def SetColumn(self,control,column):
  """
  SetColumn(self: TableLayoutSettings,control: object,column: int)

   Sets the column position for the specified child control.

  

   control: A control contained within a cell.

   column: The column position for the specified child control.
  """
  pass
 def SetColumnSpan(self,control,value):
  """
  SetColumnSpan(self: TableLayoutSettings,control: object,value: int)

   Sets the number of columns that the cell containing the child control spans.

  

   control: A control contained within a cell.

   value: The number of columns that the cell containing the child control spans.
  """
  pass
 def SetRow(self,control,row):
  """
  SetRow(self: TableLayoutSettings,control: object,row: int)

   Sets the row position of the specified child control.

  

   control: A control contained within a cell.

   row: The row position of the specified child control.
  """
  pass
 def SetRowSpan(self,control,value):
  """
  SetRowSpan(self: TableLayoutSettings,control: object,value: int)

   Sets the number of rows that the cell containing the child control spans.

  

   control: A control contained within a cell.

   value: The number of rows that the cell containing the child control spans.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __reduce_ex__(self,*args):
  pass
 ColumnCount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the maximum number of columns allowed in the table layout.



Get: ColumnCount(self: TableLayoutSettings) -> int



Set: ColumnCount(self: TableLayoutSettings)=value

"""

 ColumnStyles=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of styles used to determine the look and feel of the table layout columns.



Get: ColumnStyles(self: TableLayoutSettings) -> TableLayoutColumnStyleCollection



"""

 GrowStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating how the table layout should expand to accommodate new cells when all existing cells are occupied.



Get: GrowStyle(self: TableLayoutSettings) -> TableLayoutPanelGrowStyle



Set: GrowStyle(self: TableLayoutSettings)=value

"""

 LayoutEngine=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the current table layout engine.



Get: LayoutEngine(self: TableLayoutSettings) -> LayoutEngine



"""

 RowCount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the maximum number of rows allowed in the table layout.



Get: RowCount(self: TableLayoutSettings) -> int



Set: RowCount(self: TableLayoutSettings)=value

"""

 RowStyles=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of styles used to determine the look and feel of the table layout rows.



Get: RowStyles(self: TableLayoutSettings) -> TableLayoutRowStyleCollection



"""





"""Base classes for pyRevit extension components."""
import os
import os.path as op
import json
import codecs

from pyrevit import PyRevitException, HOST_APP
from pyrevit.compat import safe_strtype
from pyrevit import framework
from pyrevit import coreutils
from pyrevit.coreutils.logger import get_logger
import pyrevit.extensions as exts
from pyrevit.extensions.genericcomps import GenericComponent
from pyrevit.extensions.genericcomps import GenericUIContainer
from pyrevit.extensions.genericcomps import GenericUICommand
from pyrevit import versionmgr


# pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)


EXT_HASH_VALUE_KEY = 'dir_hash_value'
EXT_HASH_VERSION_KEY = 'pyrvt_version'


# Derived classes here correspond to similar elements in Revit ui.
# Under Revit UI:
# Packages contain Tabs, Tabs contain, Panels, Panels contain Stacks,
# Commands, or Command groups
# ------------------------------------------------------------------------------
class NoButton(GenericUICommand):
    """This is not a button."""
    type_id = exts.NOGUI_COMMAND_POSTFIX


class NoScriptButton(GenericUICommand):
    """Base for buttons that doesn't run a script."""
    def __init__(self, cmp_path=None, needs_commandclass=False):
        # using classname otherwise exceptions in superclasses won't show
        GenericUICommand.__init__(self, cmp_path=cmp_path, needs_script=False)
        self.assembly = self.command_class = self.avail_command_class = None
        # read metadata from metadata file
        if self.meta:
            # get the target assembly from metadata
            self.assembly = \
                self.meta.get(exts.MDATA_LINK_BUTTON_ASSEMBLY, None)

            # get the target command class from metadata
            self.command_class = \
                self.meta.get(exts.MDATA_LINK_BUTTON_COMMAND_CLASS, None)

            # get the target command class from metadata
            self.avail_command_class = \
                self.meta.get(exts.MDATA_LINK_BUTTON_AVAIL_COMMAND_CLASS, None)

            # for invoke buttons there is no script source so
            # assign the metadata file to the script
            self.script_file = self.config_script_file = self.meta_file
        else:
            mlogger.debug("%s does not specify target assembly::class.", self)

        if self.directory and not self.assembly:
            mlogger.error("%s does not specify target assembly.", self)

        if self.directory and needs_commandclass and not self.command_class:
            mlogger.error("%s does not specify target command class.", self)

        mlogger.debug('%s assembly.class: %s.%s',
                      self, self.assembly, self.command_class)

    def get_target_assembly(self, required=False):
        assm_file = self.assembly.lower()
        if not assm_file.endswith(framework.ASSEMBLY_FILE_TYPE):
            assm_file += '.' + framework.ASSEMBLY_FILE_TYPE

        # try finding assembly for this specific host version
        target_asm_by_host = self.find_bundle_module(assm_file, by_host=True)
        if target_asm_by_host:
            return target_asm_by_host
        # try find assembly by its name
        target_asm = self.find_bundle_module(assm_file)
        if target_asm:
            return target_asm

        if required:
            mlogger.error("%s can not find target assembly.", self)

        return ''


class LinkButton(NoScriptButton):
    """Link button."""
    type_id = exts.LINK_BUTTON_POSTFIX

    def __init__(self, cmp_path=None):
        # using classname otherwise exceptions in superclasses won't show
        NoScriptButton.__init__(
            self,
            cmp_path=cmp_path,
            needs_commandclass=True
            )

        if self.context:
            mlogger.warn(
                "Linkbutton bundles do not support \"context:\". "
                "Use \"availability_class:\" instead and specify name of "
                "availability class in target assembly | %s", self
                )
            self.context = None


class InvokeButton(NoScriptButton):
    """Invoke button."""
    type_id = exts.INVOKE_BUTTON_POSTFIX

    def __init__(self, cmp_path=None):
        # using classname otherwise exceptions in superclasses won't show
        NoScriptButton.__init__(self, cmp_path=cmp_path)


class PushButton(GenericUICommand):
    """Push button."""
    type_id = exts.PUSH_BUTTON_POSTFIX


class PanelPushButton(GenericUICommand):
    """Panel push button."""
    type_id = exts.PANEL_PUSH_BUTTON_POSTFIX


class SmartButton(GenericUICommand):
    """Smart button."""
    type_id = exts.SMART_BUTTON_POSTFIX


class ContentButton(GenericUICommand):
    """Content Button."""
    type_id = exts.CONTENT_BUTTON_POSTFIX

    def __init__(self, cmp_path=None):
        # using classname otherwise exceptions in superclasses won't show
        GenericUICommand.__init__(
            self,
            cmp_path=cmp_path,
            needs_script=False
            )

        # Initialize content paths
        self.content = None
        self.content_alt = None

        # Try to get content file from metadata first
        if self.meta:
            content_from_meta = self.meta.get(exts.MDATA_CONTENT, None)
            if content_from_meta:
                resolved_path = self._resolve_content_path(content_from_meta)
                if resolved_path:
                    self.script_file = resolved_path
                else:
                    mlogger.error('Content file specified in metadata not found: %s',
                                 content_from_meta)

            alt_content_from_meta = self.meta.get(exts.MDATA_CONTENT_ALT, None)
            if alt_content_from_meta:
                resolved_alt_path = self._resolve_content_path(alt_content_from_meta)
                if resolved_alt_path:
                    self.config_script_file = resolved_alt_path
                else:
                    mlogger.error('Alternative content file specified in metadata not found: %s',
                                 alt_content_from_meta)

        # Fall back to naming convention if not found in metadata
        # find content file
        if not self.script_file:
            self.script_file = \
                self.find_bundle_file([
                    exts.CONTENT_VERSION_POSTFIX.format(
                        version=HOST_APP.version
                        ),
                    ])
            if not self.script_file:
                self.script_file = \
                    self.find_bundle_file([
                        exts.CONTENT_POSTFIX,
                        ])
        # requires at least one bundles
        if self.directory and not self.script_file:
            mlogger.error('Command %s: Does not have content file.', self)
            self.script_file = ''

        # find alternative content file
        if not self.config_script_file:
            self.config_script_file = \
                self.find_bundle_file([
                    exts.ALT_CONTENT_VERSION_POSTFIX.format(
                        version=HOST_APP.version
                        ),
                    ])
            if not self.config_script_file:
                self.config_script_file = \
                    self.find_bundle_file([
                        exts.ALT_CONTENT_POSTFIX,
                        ])
        if not self.config_script_file:
            self.config_script_file = self.script_file

    def _resolve_content_path(self, path):
        # Check if it's an absolute path
        if op.isabs(path):
            if op.exists(path):
                if not path.lower().endswith(exts.CONTENT_FILE_FORMAT):
                    mlogger.error('Content file must be a Revit family (.rfa): %s',
                                 path)
                    return None
                return path
            else:
                mlogger.error('Content file specified in metadata not found: %s',
                             path)
                return None
        
        # Treat as relative to bundle directory
        if self.directory:
            # Normalize the path to handle .. and . properly
            bundle_path = op.normpath(op.join(self.directory, path))
            if op.exists(bundle_path):
                if not bundle_path.lower().endswith(exts.CONTENT_FILE_FORMAT):
                    mlogger.error('Content file must be a Revit family (.rfa): %s',
                                 bundle_path)
                    return None
                return bundle_path
            else:
                mlogger.error('Content file specified in metadata not found: %s (resolved to: %s)',
                             path, bundle_path)
                return None
        
        mlogger.error('Content file specified in metadata not found: %s (no bundle directory)',
                     path)
        return None


class URLButton(GenericUICommand):
    """URL button."""
    type_id = exts.URL_BUTTON_POSTFIX

    def __init__(self, cmp_path=None):
        # using classname otherwise exceptions in superclasses won't show
        GenericUICommand.__init__(self, cmp_path=cmp_path, needs_script=False)
        self.target_url = None
        # read metadata from metadata file
        if self.meta:
            # get the target url from metadata
            self.target_url = \
                self.meta.get(exts.MDATA_URL_BUTTON_HYPERLINK, None)
            # for url buttons there is no script source so
            # assign the metadata file to the script
            self.script_file = self.config_script_file = self.meta_file
        else:
            mlogger.debug("%s does not specify target assembly::class.", self)

        if self.directory and not self.target_url:
            mlogger.error("%s does not specify target url.", self)

        mlogger.debug('%s target url: %s', self, self.target_url)

    def get_target_url(self):
        return self.target_url or ""


class GenericUICommandGroup(GenericUIContainer):
    """Generic UI command group.

    Command groups only include commands.
    These classes can include GenericUICommand as sub components.
    """
    allowed_sub_cmps = [GenericUICommand, NoScriptButton]

    @property
    def control_id(self):
        # stacks don't have control id
        if self.parent_ctrl_id:
            deepend_parent_id = self.parent_ctrl_id.replace(
                '_%CustomCtrl',
                '_%CustomCtrl_%CustomCtrl'
            )
            return deepend_parent_id + '%{}'.format(self.name)
        else:
            return '%{}%'.format(self.name)

    def has_commands(self):
        for component in self:
            if isinstance(component, GenericUICommand):
                return True


class PullDownButtonGroup(GenericUICommandGroup):
    """Pulldown button group."""
    type_id = exts.PULLDOWN_BUTTON_POSTFIX


class SplitPushButtonGroup(GenericUICommandGroup):
    """Split push button group."""
    type_id = exts.SPLITPUSH_BUTTON_POSTFIX


class SplitButtonGroup(GenericUICommandGroup):
    """Split button group."""
    type_id = exts.SPLIT_BUTTON_POSTFIX


class GenericStack(GenericUIContainer):
    """Generic UI stack.

    Stacks include GenericUICommand, or GenericUICommandGroup.
    """
    type_id = exts.STACK_BUTTON_POSTFIX

    allowed_sub_cmps = \
        [GenericUICommandGroup, GenericUICommand, NoScriptButton]

    @property
    def control_id(self):
        # stacks don't have control id
        return self.parent_ctrl_id if self.parent_ctrl_id else ''

    def has_commands(self):
        for component in self:
            if not component.is_container:
                if isinstance(component, GenericUICommand):
                    return True
            else:
                if component.has_commands():
                    return True


class StackButtonGroup(GenericStack):
    """Stack buttons group."""
    type_id = exts.STACK_BUTTON_POSTFIX


class Panel(GenericUIContainer):
    """Panel container.

    Panels include GenericStack, GenericUICommand, or GenericUICommandGroup
    """
    type_id = exts.PANEL_POSTFIX
    allowed_sub_cmps = \
        [GenericStack, GenericUICommandGroup, GenericUICommand, NoScriptButton]

    def __init__(self, cmp_path=None):
        # using classname otherwise exceptions in superclasses won't show
        GenericUIContainer.__init__(self, cmp_path=cmp_path)
        self.panel_background = \
            self.title_background = \
                self.slideout_background = None
        # read metadata from metadata file
        if self.meta:
            # check for background color configs
            self.panel_background = \
                self.meta.get(exts.MDATA_BACKGROUND_KEY, None)
            if self.panel_background:
                if isinstance(self.panel_background, dict):
                    self.title_background = self.panel_background.get(
                        exts.MDATA_BACKGROUND_TITLE_KEY, None)
                    self.slideout_background = self.panel_background.get(
                        exts.MDATA_BACKGROUND_SLIDEOUT_KEY, None)
                    self.panel_background = self.panel_background.get(
                        exts.MDATA_BACKGROUND_PANEL_KEY, None)
                elif not isinstance(self.panel_background, str):
                    mlogger.error(
                        "%s bad background definition in metadata.", self)

    def has_commands(self):
        for component in self:
            if not component.is_container:
                if isinstance(component, GenericUICommand):
                    return True
            else:
                if component.has_commands():
                    return True

    def contains(self, item_name):
        # Panels contain stacks. But stacks itself does not have any ui and its
        # subitems are displayed within the ui of the parent panel.
        # This is different from pulldowns and other button groups.
        # Button groups, contain and display their sub components in their
        # own drop down menu. So when checking if panel has a button,
        # panel should check all the items visible to the user and respond.
        item_exists = GenericUIContainer.contains(self, item_name)
        if item_exists:
            return True
        else:
            # if child is a stack item, check its children too
            for component in self:
                if isinstance(component, GenericStack) \
                        and component.contains(item_name):
                    return True


class Tab(GenericUIContainer):
    """Tab container for Panels."""
    type_id = exts.TAB_POSTFIX
    allowed_sub_cmps = [Panel]

    def has_commands(self):
        for panel in self:
            if panel.has_commands():
                return True
        return False


class Extension(GenericUIContainer):
    """UI Tools extension."""
    type_id = exts.ExtensionTypes.UI_EXTENSION.POSTFIX
    allowed_sub_cmps = [Tab]

    def __init__(self, cmp_path=None):
        self.pyrvt_version = None
        self.dir_hash_value = None
        # using classname otherwise exceptions in superclasses won't show
        GenericUIContainer.__init__(self, cmp_path=cmp_path)

    def _calculate_extension_dir_hash(self):
        """Creates a unique hash # to represent state of directory."""
        # search does not include png files:
        #   if png files are added the parent folder mtime gets affected
        #   cache only saves the png address and not the contents so they'll
        #   get loaded everytime
        #       see http://stackoverflow.com/a/5141710/2350244
        pat = '(\\' + exts.TAB_POSTFIX + ')|(\\' + exts.PANEL_POSTFIX + ')'
        pat += '|(\\' + exts.PULLDOWN_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.SPLIT_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.SPLITPUSH_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.STACK_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.PUSH_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.SMART_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.LINK_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.PANEL_PUSH_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.PANEL_PUSH_BUTTON_POSTFIX + ')'
        pat += '|(\\' + exts.CONTENT_BUTTON_POSTFIX + ')'
        # tnteresting directories
        pat += '|(\\' + exts.COMP_LIBRARY_DIR_NAME + ')'
        pat += '|(\\' + exts.COMP_HOOKS_DIR_NAME + ')'
        # search for scripts, setting files (future support), and layout files
        patfile = '(\\' + exts.PYTHON_SCRIPT_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.CSHARP_SCRIPT_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.VB_SCRIPT_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.RUBY_SCRIPT_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.DYNAMO_SCRIPT_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.GRASSHOPPER_SCRIPT_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.GRASSHOPPERX_SCRIPT_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.CONTENT_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.YAML_FILE_FORMAT + ')'
        patfile += '|(\\' + exts.JSON_FILE_FORMAT + ')'
        from pyrevit.revit import ui
        return coreutils.calculate_dir_hash(self.directory, pat, patfile) + str(ui.get_current_theme())

    def _update_from_directory(self):   #pylint: disable=W0221
        # using classname otherwise exceptions in superclasses won't show
        GenericUIContainer._update_from_directory(self)
        self.pyrvt_version = versionmgr.get_pyrevit_version().get_formatted()

        # extensions can store event hooks under
        # hooks/ inside the component folder
        hooks_path = op.join(self.directory, exts.COMP_HOOKS_DIR_NAME)
        self.hooks_path = hooks_path if op.exists(hooks_path) else None

        # extensions can store preflight checks under
        # checks/ inside the component folder
        checks_path = op.join(self.directory, exts.COMP_CHECKS_DIR_NAME)
        self.checks_path = checks_path if op.exists(checks_path) else None

        self.dir_hash_value = self._calculate_extension_dir_hash()

    @property
    def control_id(self):
        return None

    @property
    def startup_script(self):
        return self.find_bundle_file([
            exts.PYTHON_EXT_STARTUP_FILE,
            exts.CSHARP_EXT_STARTUP_FILE,
            exts.VB_EXT_STARTUP_FILE,
            exts.RUBY_EXT_STARTUP_FILE,
        ])

    def get_hash(self):
        return coreutils.get_str_hash(safe_strtype(self.get_cache_data()))

    def get_all_commands(self):
        return self.find_components_of_type(GenericUICommand)

    def get_manifest_file(self):
        return self.get_bundle_file(exts.EXT_MANIFEST_FILE)

    def get_manifest(self):
        manifest_file = self.get_manifest_file()
        if manifest_file:
            with codecs.open(manifest_file, 'r', 'utf-8') as mfile:
                try:
                    manifest_cfg = json.load(mfile)
                    return manifest_cfg
                except Exception as manfload_err:
                    print('Can not parse ext manifest file: {} '
                          '| {}'.format(manifest_file, manfload_err))
                    return

    def configure(self):
        cfg_dict = self.get_manifest()
        if cfg_dict:
            for component in self:
                component.configure(cfg_dict)

    def get_extension_modules(self):
        modules = []
        if self.binary_path and op.exists(self.binary_path):
            for item in os.listdir(self.binary_path):
                item_path = op.join(self.binary_path, item)
                item_name = item.lower()
                if op.isfile(item_path) \
                        and item_name.endswith(framework.ASSEMBLY_FILE_TYPE):
                    modules.append(item_path)
        return modules

    def get_command_modules(self):
        referenced_modules = set()
        for cmd in self.get_all_commands():
            for module in cmd.modules:
                cmd_module = cmd.find_bundle_module(module)
                if cmd_module:
                    referenced_modules.add(cmd_module)
        return referenced_modules

    def get_hooks(self):
        hook_scripts = os.listdir(self.hooks_path) if self.hooks_path else []
        return [op.join(self.hooks_path, x) for x in hook_scripts]

    def get_checks(self):
        check_scripts = os.listdir(self.checks_path) if self.checks_path else []
        return [op.join(self.checks_path, x) for x in check_scripts]


class LibraryExtension(GenericComponent):
    """Library extension."""
    type_id = exts.ExtensionTypes.LIB_EXTENSION.POSTFIX

    def __init__(self, cmp_path=None):
        # using classname otherwise exceptions in superclasses won't show
        GenericComponent.__init__(self)
        self.directory = cmp_path

        if self.directory:
            self.name = op.splitext(op.basename(self.directory))[0]

    def __repr__(self):
        return '<type_id \'{}\' name \'{}\' @ \'{}\'>'\
            .format(self.type_id, self.name, self.directory)

    @classmethod
    def matches(cls, component_path):
        return component_path.lower().endswith(cls.type_id)



class ProvidePropertyAttribute(Attribute,_Attribute):
 """
 Specifies the name of the property that an implementer of System.ComponentModel.IExtenderProvider offers to other components. This class cannot be inherited

 

 ProvidePropertyAttribute(propertyName: str,receiverType: Type)

 ProvidePropertyAttribute(propertyName: str,receiverTypeName: str)
 """
 def Equals(self,obj):
  """
  Equals(self: ProvidePropertyAttribute,obj: object) -> bool

  

   Returns whether the value of the given object is equal to the current 

    System.ComponentModel.ProvidePropertyAttribute.

  

  

   obj: The object to test the value equality of.

   Returns: true if the value of the given object is equal to that of the current; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: ProvidePropertyAttribute) -> int

  

   Returns the hash code for this instance.

   Returns: A hash code for the current System.ComponentModel.ProvidePropertyAttribute.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,propertyName,*__args):
  """
  __new__(cls: type,propertyName: str,receiverType: Type)

  __new__(cls: type,propertyName: str,receiverTypeName: str)
  """
  pass
 def __ne__(self,*args):
  pass
 PropertyName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of a property that this class provides.



Get: PropertyName(self: ProvidePropertyAttribute) -> str



"""

 ReceiverTypeName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the data type this property can extend.



Get: ReceiverTypeName(self: ProvidePropertyAttribute) -> str



"""

 TypeId=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a unique identifier for this attribute.



Get: TypeId(self: ProvidePropertyAttribute) -> object



"""





class Bitmap(Image,ISerializable,ICloneable,IDisposable):
 """
 Encapsulates a GDI+ bitmap,which consists of the pixel data for a graphics image and its attributes. A System.Drawing.Bitmap is an object used to work with images defined by pixel data.

 

 Bitmap(filename: str)

 Bitmap(filename: str,useIcm: bool)

 Bitmap(type: Type,resource: str)

 Bitmap(stream: Stream)

 Bitmap(stream: Stream,useIcm: bool)

 Bitmap(width: int,height: int,stride: int,format: PixelFormat,scan0: IntPtr)

 Bitmap(width: int,height: int,format: PixelFormat)

 Bitmap(width: int,height: int)

 Bitmap(width: int,height: int,g: Graphics)

 Bitmap(original: Image)

 Bitmap(original: Image,width: int,height: int)

 Bitmap(original: Image,newSize: Size)
 """
 def Clone(self,rect=None,format=None):
  """
  Clone(self: Bitmap,rect: RectangleF,format: PixelFormat) -> Bitmap

  

   Creates a copy of the section of this System.Drawing.Bitmap defined with a specified 

    System.Drawing.Imaging.PixelFormat enumeration.

  

  

   rect: Defines the portion of this System.Drawing.Bitmap to copy.

   format: Specifies the System.Drawing.Imaging.PixelFormat enumeration for the destination 

    System.Drawing.Bitmap.

  

   Returns: The System.Drawing.Bitmap that this method creates.

  Clone(self: Bitmap,rect: Rectangle,format: PixelFormat) -> Bitmap

  

   Creates a copy of the section of this System.Drawing.Bitmap defined by System.Drawing.Rectangle 

    structure and with a specified System.Drawing.Imaging.PixelFormat enumeration.

  

  

   rect: Defines the portion of this System.Drawing.Bitmap to copy. Coordinates are relative to this 

    System.Drawing.Bitmap.

  

   format: The pixel format for the new System.Drawing.Bitmap. This must specify a value that begins with 

    Format.

  

   Returns: The new System.Drawing.Bitmap that this method creates.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Image,disposing: bool)

   Releases the unmanaged resources used by the System.Drawing.Image and optionally releases the 

    managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 @staticmethod
 def FromHicon(hicon):
  """
  FromHicon(hicon: IntPtr) -> Bitmap

  

   Creates a System.Drawing.Bitmap from a Windows handle to an icon.

  

   hicon: A handle to an icon.

   Returns: The System.Drawing.Bitmap that this method creates.
  """
  pass
 @staticmethod
 def FromResource(hinstance,bitmapName):
  """
  FromResource(hinstance: IntPtr,bitmapName: str) -> Bitmap

  

   Creates a System.Drawing.Bitmap from the specified Windows resource.

  

   hinstance: A handle to an instance of the executable file that contains the resource.

   bitmapName: A string that contains the name of the resource bitmap.

   Returns: The System.Drawing.Bitmap that this method creates.
  """
  pass
 def GetHbitmap(self,background=None):
  """
  GetHbitmap(self: Bitmap,background: Color) -> IntPtr

  

   Creates a GDI bitmap object from this System.Drawing.Bitmap.

  

   background: A System.Drawing.Color structure that specifies the background color. This parameter is ignored 

    if the bitmap is totally opaque.

  

   Returns: A handle to the GDI bitmap object that this method creates.

  GetHbitmap(self: Bitmap) -> IntPtr

  

   Creates a GDI bitmap object from this System.Drawing.Bitmap.

   Returns: A handle to the GDI bitmap object that this method creates.
  """
  pass
 def GetHicon(self):
  """
  GetHicon(self: Bitmap) -> IntPtr

  

   Returns the handle to an icon.

   Returns: A Windows handle to an icon with the same image as the System.Drawing.Bitmap.
  """
  pass
 def GetPixel(self,x,y):
  """
  GetPixel(self: Bitmap,x: int,y: int) -> Color

  

   Gets the color of the specified pixel in this System.Drawing.Bitmap.

  

   x: The x-coordinate of the pixel to retrieve.

   y: The y-coordinate of the pixel to retrieve.

   Returns: A System.Drawing.Color structure that represents the color of the specified pixel.
  """
  pass
 def LockBits(self,rect,flags,format,bitmapData=None):
  """
  LockBits(self: Bitmap,rect: Rectangle,flags: ImageLockMode,format: PixelFormat,bitmapData: BitmapData) -> BitmapData

  

   Locks a System.Drawing.Bitmap into system memory

  

   rect: A rectangle structure that specifies the portion of the System.Drawing.Bitmap to lock.

   flags: One of the System.Drawing.Imaging.ImageLockMode values that specifies the access level 

    (read/write) for the System.Drawing.Bitmap.

  

   format: One of the System.Drawing.Imaging.PixelFormat values that specifies the data format of the 

    System.Drawing.Bitmap.

  

   bitmapData: A System.Drawing.Imaging.BitmapData that contains information about the lock operation.

   Returns: A System.Drawing.Imaging.BitmapData that contains information about the lock operation.

  LockBits(self: Bitmap,rect: Rectangle,flags: ImageLockMode,format: PixelFormat) -> BitmapData

  

   Locks a System.Drawing.Bitmap into system memory.

  

   rect: A System.Drawing.Rectangle structure that specifies the portion of the System.Drawing.Bitmap to 

    lock.

  

   flags: An System.Drawing.Imaging.ImageLockMode enumeration that specifies the access level (read/write) 

    for the System.Drawing.Bitmap.

  

   format: A System.Drawing.Imaging.PixelFormat enumeration that specifies the data format of this 

    System.Drawing.Bitmap.

  

   Returns: A System.Drawing.Imaging.BitmapData that contains information about this lock operation.
  """
  pass
 def MakeTransparent(self,transparentColor=None):
  """
  MakeTransparent(self: Bitmap,transparentColor: Color)

   Makes the specified color transparent for this System.Drawing.Bitmap.

  

   transparentColor: The System.Drawing.Color structure that represents the color to make transparent.

  MakeTransparent(self: Bitmap)

   Makes the default transparent color transparent for this System.Drawing.Bitmap.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def SetPixel(self,x,y,color):
  """
  SetPixel(self: Bitmap,x: int,y: int,color: Color)

   Sets the color of the specified pixel in this System.Drawing.Bitmap.

  

   x: The x-coordinate of the pixel to set.

   y: The y-coordinate of the pixel to set.

   color: A System.Drawing.Color structure that represents the color to assign to the specified pixel.
  """
  pass
 def SetResolution(self,xDpi,yDpi):
  """
  SetResolution(self: Bitmap,xDpi: Single,yDpi: Single)

   Sets the resolution for this System.Drawing.Bitmap.

  

   xDpi: The horizontal resolution,in dots per inch,of the System.Drawing.Bitmap.

   yDpi: The vertical resolution,in dots per inch,of the System.Drawing.Bitmap.
  """
  pass
 def UnlockBits(self,bitmapdata):
  """
  UnlockBits(self: Bitmap,bitmapdata: BitmapData)

   Unlocks this System.Drawing.Bitmap from system memory.

  

   bitmapdata: A System.Drawing.Imaging.BitmapData that specifies information about the lock operation.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,filename: str)

  __new__(cls: type,filename: str,useIcm: bool)

  __new__(cls: type,type: Type,resource: str)

  __new__(cls: type,stream: Stream)

  __new__(cls: type,stream: Stream,useIcm: bool)

  __new__(cls: type,width: int,height: int,stride: int,format: PixelFormat,scan0: IntPtr)

  __new__(cls: type,width: int,height: int,format: PixelFormat)

  __new__(cls: type,width: int,height: int)

  __new__(cls: type,width: int,height: int,g: Graphics)

  __new__(cls: type,original: Image)

  __new__(cls: type,original: Image,width: int,height: int)

  __new__(cls: type,original: Image,newSize: Size)
  """
  pass
 def __reduce_ex__(self,*args):
  pass



"""Idempotent operations to ensure a database object exists."""
import os.path as op

from System import Int64

from pyrevit import HOST_APP, DOCS, PyRevitException
from pyrevit import coreutils
from pyrevit.coreutils.logger import get_logger
from pyrevit import DB
from pyrevit.compat import get_elementid_from_value_func
from pyrevit.revit.db import query
from pyrevit.revit.db import create
from pyrevit.revit.db import transaction    #pylint: disable=E0611


#pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)
get_elementid_from_value = get_elementid_from_value_func()

def ensure_sharedparam(sparam_name, sparam_categories, sparam_group,
                       load_param=True, allow_vary_betwen_groups=False,
                       doc=None):
    doc = doc or DOCS.doc
    if query.model_has_parameter(sparam_name, doc=doc):
        if allow_vary_betwen_groups:
            param = query.get_project_parameter(sparam_name, doc=doc)
            if isinstance(param.param_def, DB.InternalDefinition) \
                    and not param.param_def.VariesAcrossGroups:
                param.param_def.SetAllowVaryBetweenGroups(doc, True)
        return True
    elif load_param:
        create.create_shared_param(
            sparam_name,
            sparam_categories,
            sparam_group,
            doc=doc,
            allow_vary_betwen_groups=allow_vary_betwen_groups
            )
        return True


def ensure_sharedparam_file(spfilepath):
    if spfilepath and not HOST_APP.app.SharedParametersFilename:
        HOST_APP.app.SharedParametersFilename = spfilepath
    elif HOST_APP.app.SharedParametersFilename \
            and not op.normpath(HOST_APP.app.SharedParametersFilename) \
                == op.normpath(spfilepath):
        HOST_APP.app.SharedParametersFilename = spfilepath

    return HOST_APP.app.OpenSharedParameterFile()


def ensure_family(family_name, family_file, doc=None):
    doc = doc or DOCS.doc
    famsym = query.get_family(family_name, doc=doc)
    if not famsym:
        with transaction.Transaction('Load Family', doc=doc):
            mlogger.debug('Family \"%s\" did not exist.', family_name)
            if create.load_family(family_file, doc=doc):
                return query.get_family(family_name, doc=doc)
            else:
                raise PyRevitException('Error loading family from: {}'
                                       .format(family_file))
    return famsym


def ensure_element_ids(mixed_list):
    element_id_list = []

    if not hasattr(mixed_list, '__iter__'):
        mixed_list = [mixed_list]

    for item in mixed_list:
        if isinstance(item, DB.ElementId):
            element_id_list.append(item)
        elif isinstance(item, DB.Element):
            element_id_list.append(item.Id)
        elif hasattr(item, 'Id') and isinstance(item.Id, DB.ElementId):
            element_id_list.append(item.Id)
        elif isinstance(item, (int, Int64)):
            element_id_list.append(get_elementid_from_value(item))

    return element_id_list


def ensure_workset(workset_name, partial=False, doc=None):
    doc = doc or DOCS.doc
    workset = query.model_has_workset(workset_name, partial=partial, doc=doc)
    if workset:
        return workset
    else:
        return create.create_workset(workset_name, doc=doc)


def ensure_text_type(name,
                     font_name=None,
                     font_size=0.01042,
                     tab_size=0.02084,
                     bold=False,
                     italic=False,
                     underline=False,
                     width_factor=1.0,
                     doc=None):
    doc = doc or DOCS.doc
    # check if type exists
    for ttype in query.get_types_by_class(DB.TextNoteType, doc=doc):
        if query.get_name(ttype) == name:
            return ttype
    # otherwise create it
    return create.create_text_type(
        name,
        font_name=font_name,
        font_size=font_size,
        tab_size=tab_size,
        bold=bold,
        italic=italic,
        underline=underline,
        width_factor=width_factor,
        doc=doc)


def revision_has_numbertype(revision):
    doc = revision.Document
    none_numtype = coreutils.get_enum_none(DB.RevisionNumberType)
    if HOST_APP.is_newer_than(2022):
        numbering = doc.GetElement(revision.RevisionNumberingSequenceId)
        if numbering:
            return numbering.NumberType != none_numtype
    else:
        return revision.NumberType != none_numtype



class ActivationContext(object,IDisposable,ISerializable):
 """ Identifies the activation context for the current application. This class cannot be inherited. """
 @staticmethod
 def CreatePartialActivationContext(identity,manifestPaths=None):
  """
  CreatePartialActivationContext(identity: ApplicationIdentity,manifestPaths: Array[str]) -> ActivationContext

  

   Initializes a new instance of the System.ActivationContext class using the specified application 

    identity and array of manifest paths.

  

  

   identity: An object that identifies an application.

   manifestPaths: A string array of manifest paths for the application.

   Returns: An object with the specified application identity and array of manifest paths.

  CreatePartialActivationContext(identity: ApplicationIdentity) -> ActivationContext

  

   Initializes a new instance of the System.ActivationContext class using the specified application 

    identity.

  

  

   identity: An object that identifies an application.

   Returns: An object with the specified application identity.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: ActivationContext)

   Releases all resources used by the System.ActivationContext.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 ApplicationManifestBytes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the ClickOnce application manifest for the current application.



Get: ApplicationManifestBytes(self: ActivationContext) -> Array[Byte]



"""

 DeploymentManifestBytes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the ClickOnce deployment manifest for the current application.



Get: DeploymentManifestBytes(self: ActivationContext) -> Array[Byte]



"""

 Form=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the form,or store context,for the current application.



Get: Form(self: ActivationContext) -> ContextForm



"""

 Identity=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the application identity for the current application.



Get: Identity(self: ActivationContext) -> ApplicationIdentity



"""


 ContextForm=None




class RowDefinitionCollection(object,IList[RowDefinition],ICollection[RowDefinition],IEnumerable[RowDefinition],IEnumerable,IList,ICollection):
 """ Provides access to an ordered,strongly typed collection of System.Windows.Controls.RowDefinition objects. """
 def Add(self,value):
  """
  Add(self: RowDefinitionCollection,value: RowDefinition)

   Adds a System.Windows.Controls.RowDefinition element to a 

    System.Windows.Controls.RowDefinitionCollection.

  

  

   value: Identifies the System.Windows.Controls.RowDefinition to add to the collection.
  """
  pass
 def Clear(self):
  """
  Clear(self: RowDefinitionCollection)

   Clears the content of the System.Windows.Controls.RowDefinitionCollection.
  """
  pass
 def Contains(self,value):
  """
  Contains(self: RowDefinitionCollection,value: RowDefinition) -> bool

  

   Determines whether a given System.Windows.Controls.RowDefinition exists within a 

    System.Windows.Controls.RowDefinitionCollection.

  

  

   value: Identifies the System.Windows.Controls.RowDefinition that is being tested.

   Returns: true if the System.Windows.Controls.RowDefinition exists within the collection; otherwise false.
  """
  pass
 def CopyTo(self,array,index):
  """
  CopyTo(self: RowDefinitionCollection,array: Array[RowDefinition],index: int)

   Copies an array of System.Windows.Controls.RowDefinition objects to a given index position 

    within a System.Windows.Controls.RowDefinitionCollection.

  

  

   array: An array of System.Windows.Controls.RowDefinition objects.

   index: Identifies the index position within array to which the System.Windows.Controls.RowDefinition 

    objects are copied.
  """
  pass
 def IndexOf(self,value):
  """
  IndexOf(self: RowDefinitionCollection,value: RowDefinition) -> int

  

   Returns the index position of a given System.Windows.Controls.RowDefinition within a 

    System.Windows.Controls.RowDefinitionCollection.

  

  

   value: The System.Windows.Controls.RowDefinition whose index position is desired.

   Returns: The index of value if found in the collection; otherwise,-1.
  """
  pass
 def Insert(self,index,value):
  """
  Insert(self: RowDefinitionCollection,index: int,value: RowDefinition)

   Inserts a System.Windows.Controls.RowDefinition at the specified index position within a 

    System.Windows.Controls.RowDefinitionCollection.

  

  

   index: The position within the collection where the item is inserted.

   value: The System.Windows.Controls.RowDefinition to insert.
  """
  pass
 def Remove(self,value):
  """
  Remove(self: RowDefinitionCollection,value: RowDefinition) -> bool

  

   Removes a System.Windows.Controls.RowDefinition from a 

    System.Windows.Controls.RowDefinitionCollection.

  

  

   value: The System.Windows.Controls.RowDefinition to remove from the collection.

   Returns: true if the System.Windows.Controls.RowDefinition was found in the collection and removed; 

    otherwise,false.
  """
  pass
 def RemoveAt(self,index):
  """
  RemoveAt(self: RowDefinitionCollection,index: int)

   Removes a System.Windows.Controls.RowDefinition from a 

    System.Windows.Controls.RowDefinitionCollection at the specified index position.

  

  

   index: The position within the collection at which the System.Windows.Controls.RowDefinition is removed.
  """
  pass
 def RemoveRange(self,index,count):
  """
  RemoveRange(self: RowDefinitionCollection,index: int,count: int)

   Removes a range of System.Windows.Controls.RowDefinition objects from a 

    System.Windows.Controls.RowDefinitionCollection.

  

  

   index: The position within the collection at which the first System.Windows.Controls.RowDefinition is 

    removed.

  

   count: The total number of System.Windows.Controls.RowDefinition objects to remove from the collection.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+y """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: ICollection[RowDefinition],item: RowDefinition) -> bool

  __contains__(self: IList,value: object) -> bool

  

   Determines whether the System.Collections.IList contains a specific value.

  

   value: The object to locate in the System.Collections.IList.

   Returns: true if the System.Object is found in the System.Collections.IList; otherwise,false.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __setitem__(self,*args):
  """ x.__setitem__(i,y) <==> x[i]= """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the total number of items within this instance of System.Windows.Controls.RowDefinitionCollection.



Get: Count(self: RowDefinitionCollection) -> int



"""

 IsReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a System.Windows.Controls.RowDefinitionCollection is read-only.



Get: IsReadOnly(self: RowDefinitionCollection) -> bool



"""

 IsSynchronized=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether access to this System.Windows.Controls.RowDefinitionCollection is synchronized (thread-safe).



Get: IsSynchronized(self: RowDefinitionCollection) -> bool



"""

 SyncRoot=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an object that can be used to synchronize access to the System.Windows.Controls.RowDefinitionCollection.



Get: SyncRoot(self: RowDefinitionCollection) -> object



"""





class DnsPermissionAttribute(CodeAccessSecurityAttribute,_Attribute):
 """
 Specifies permission to request information from Domain Name Servers.

 

 DnsPermissionAttribute(action: SecurityAction)
 """
 def CreatePermission(self):
  """
  CreatePermission(self: DnsPermissionAttribute) -> IPermission

  

   Creates and returns a new instance of the System.Net.DnsPermission class.

   Returns: A System.Net.DnsPermission that corresponds to the security declaration.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,action):
  """ __new__(cls: type,action: SecurityAction) """
  pass
 def __reduce_ex__(self,*args):
  pass



class SByte(object,IComparable,IFormattable,IConvertible,IComparable[SByte],IEquatable[SByte]):
 """ Represents an 8-bit signed integer. """
 def bit_length(self,*args):
  """ bit_length(value: SByte) -> int """
  pass
 def CompareTo(self,*__args):
  """
  CompareTo(self: SByte,value: SByte) -> int

  

   Compares this instance to a specified 8-bit signed integer and returns an indication of their 

    relative values.

  

  

   value: An 8-bit signed integer to compare.

   Returns: A signed integer that indicates the relative order of this instance and value.Return Value 

    Description Less than zero This instance is less than value. Zero This instance is equal to 

    value. Greater than zero This instance is greater than value.

  

  CompareTo(self: SByte,obj: object) -> int

  

   Compares this instance to a specified object and returns an indication of their relative values.

  

   obj: An object to compare,or null.

   Returns: A signed number indicating the relative values of this instance and obj.Return Value Description 

    Less than zero This instance is less than obj. Zero This instance is equal to obj. Greater than 

    zero This instance is greater than obj.-or- obj is null.
  """
  pass
 def conjugate(self,*args):
  """ conjugate(x: SByte) -> SByte """
  pass
 def Equals(self,obj):
  """
  Equals(self: SByte,obj: SByte) -> bool

  

   Returns a value indicating whether this instance is equal to a specified System.SByte value.

  

   obj: An System.SByte value to compare to this instance.

   Returns: true if obj has the same value as this instance; otherwise,false.

  Equals(self: SByte,obj: object) -> bool

  

   Returns a value indicating whether this instance is equal to a specified object.

  

   obj: An object to compare with this instance.

   Returns: true if obj is an instance of System.SByte and equals the value of this instance; otherwise,

    false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: SByte) -> int

  

   Returns the hash code for this instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def GetTypeCode(self):
  """
  GetTypeCode(self: SByte) -> TypeCode

  

   Returns the System.TypeCode for value type System.SByte.

   Returns: The enumerated constant,System.TypeCode.SByte.
  """
  pass
 @staticmethod
 def Parse(s,*__args):
  """
  Parse(s: str,provider: IFormatProvider) -> SByte

  

   Converts the string representation of a number in a specified culture-specific format to its 

    8-bit signed integer equivalent.

  

  

   s: A string that represents a number to convert. The string is interpreted using the 

    System.Globalization.NumberStyles.Integer style.

  

   provider: An object that supplies culture-specific formatting information about s. If provider is null,

    the thread current culture is used.

  

   Returns: An 8-bit signed integer that is equivalent to the number specified in s.

  Parse(s: str,style: NumberStyles,provider: IFormatProvider) -> SByte

  

   Converts the string representation of a number that is in a specified style and culture-specific 

    format to its 8-bit signed equivalent.

  

  

   s: A string that contains the number to convert. The string is interpreted by using the style 

    specified by style.

  

   style: A bitwise combination of the enumeration values that indicates the style elements that can be 

    present in s. A typical value to specify is System.Globalization.NumberStyles.Integer.

  

   provider: An object that supplies culture-specific formatting information about s. If provider is null,

    the thread current culture is used.

  

   Returns: An 8-bit signed byte value that is equivalent to the number specified in the s parameter.

  Parse(s: str) -> SByte

  

   Converts the string representation of a number to its 8-bit signed integer equivalent.

  

   s: A string that represents a number to convert. The string is interpreted using the 

    System.Globalization.NumberStyles.Integer style.

  

   Returns: An 8-bit signed integer that is equivalent to the number contained in the s parameter.

  Parse(s: str,style: NumberStyles) -> SByte

  

   Converts the string representation of a number in a specified style to its 8-bit signed integer 

    equivalent.

  

  

   s: A string that contains a number to convert. The string is interpreted using the style specified 

    by style.

  

   style: A bitwise combination of the enumeration values that indicates the style elements that can be 

    present in s. A typical value to specify is System.Globalization.NumberStyles.Integer.

  

   Returns: An 8-bit signed integer that is equivalent to the number specified in s.
  """
  pass
 def ToString(self,*__args):
  """
  ToString(self: SByte,format: str) -> str

  

   Converts the numeric value of this instance to its equivalent string representation,using the 

    specified format.

  

  

   format: A standard or custom numeric format string.

   Returns: The string representation of the value of this instance as specified by format.

  ToString(self: SByte,format: str,provider: IFormatProvider) -> str

  

   Converts the numeric value of this instance to its equivalent string representation using the 

    specified format and culture-specific format information.

  

  

   format: A standard or custom numeric format string.

   provider: An object that supplies culture-specific formatting information.

   Returns: The string representation of the value of this instance as specified by format and provider.

  ToString(self: SByte) -> str

  

   Converts the numeric value of this instance to its equivalent string representation.

   Returns: The string representation of the value of this instance,consisting of a negative sign if the 

    value is negative,and a sequence of digits ranging from 0 to 9 with no leading zeroes.

  

  ToString(self: SByte,provider: IFormatProvider) -> str

  

   Converts the numeric value of this instance to its equivalent string representation using the 

    specified culture-specific format information.

  

  

   provider: An object that supplies culture-specific formatting information.

   Returns: The string representation of the value of this instance,as specified by provider.
  """
  pass
 @staticmethod
 def TryParse(s,*__args):
  """
  TryParse(s: str,style: NumberStyles,provider: IFormatProvider) -> (bool,SByte)

  

   Tries to convert the string representation of a number in a specified style and culture-specific 

    format to its System.SByte equivalent,and returns a value that indicates whether the conversion 

    succeeded.

  

  

   s: A string representing a number to convert.

   style: A bitwise combination of enumeration values that indicates the permitted format of s. A typical 

    value to specify is System.Globalization.NumberStyles.Integer.

  

   provider: An object that supplies culture-specific formatting information about s.

   Returns: true if s was converted successfully; otherwise,false.

  TryParse(s: str) -> (bool,SByte)

  

   Tries to convert the string representation of a number to its System.SByte equivalent,and 

    returns a value that indicates whether the conversion succeeded.

  

  

   s: A string that contains a number to convert.

   Returns: true if s was converted successfully; otherwise,false.
  """
  pass
 def __abs__(self,*args):
  """ x.__abs__() <==> abs(x) """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+y """
  pass
 def __and__(self,*args):
  """ __and__(x: SByte,y: SByte) -> SByte """
  pass
 def __cmp__(self,*args):
  """ x.__cmp__(y) <==> cmp(x,y) """
  pass
 def __div__(self,*args):
  """ x.__div__(y) <==> x/y """
  pass
 def __float__(self,*args):
  """ __float__(x: SByte) -> float """
  pass
 def __floordiv__(self,*args):
  """ x.__floordiv__(y) <==> x//y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __hash__(self,*args):
  """ x.__hash__() <==> hash(x) """
  pass
 def __hex__(self,*args):
  """ __hex__(value: SByte) -> str """
  pass
 def __index__(self,*args):
  """ __index__(x: SByte) -> int """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __int__(self,*args):
  """ __int__(x: SByte) -> int """
  pass
 def __invert__(self,*args):
  """ __invert__(x: SByte) -> SByte """
  pass
 def __lshift__(self,*args):
  """ x.__rshift__(y) <==> x<<yx.__rshift__(y) <==> x<<y """
  pass
 def __mod__(self,*args):
  """ x.__mod__(y) <==> x%y """
  pass
 def __mul__(self,*args):
  """ x.__mul__(y) <==> x*y """
  pass
 def __neg__(self,*args):
  """ x.__neg__() <==> -x """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type,value: object) -> object

  __new__(cls: type) -> object
  """
  pass
 def __nonzero__(self,*args):
  """ __nonzero__(x: SByte) -> bool """
  pass
 def __or__(self,*args):
  """ __or__(x: SByte,y: SByte) -> SByte """
  pass
 def __pos__(self,*args):
  """ __pos__(x: SByte) -> SByte """
  pass
 def __pow__(self,*args):
  """ x.__pow__(y[,z]) <==> pow(x,y[,z]) """
  pass
 def __radd__(self,*args):
  """ __radd__(x: SByte,y: SByte) -> object """
  pass
 def __rand__(self,*args):
  """ __rand__(x: SByte,y: SByte) -> SByte """
  pass
 def __rdiv__(self,*args):
  """ __rdiv__(x: SByte,y: SByte) -> object """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(x: SByte) -> str """
  pass
 def __rfloordiv__(self,*args):
  """ __rfloordiv__(x: SByte,y: SByte) -> object """
  pass
 def __rmod__(self,*args):
  """ __rmod__(x: SByte,y: SByte) -> SByte """
  pass
 def __rmul__(self,*args):
  """ __rmul__(x: SByte,y: SByte) -> object """
  pass
 def __ror__(self,*args):
  """ __ror__(x: SByte,y: SByte) -> SByte """
  pass
 def __rpow__(self,*args):
  """ __rpow__(x: SByte,y: SByte) -> object """
  pass
 def __rshift__(self,*args):
  """ x.__rshift__(y) <==> x>>yx.__rshift__(y) <==> x>>y """
  pass
 def __rsub__(self,*args):
  """ __rsub__(x: SByte,y: SByte) -> object """
  pass
 def __rtruediv__(self,*args):
  """ __rtruediv__(x: SByte,y: SByte) -> float """
  pass
 def __rxor__(self,*args):
  """ __rxor__(x: SByte,y: SByte) -> SByte """
  pass
 def __str__(self,*args):
  pass
 def __sub__(self,*args):
  """ x.__sub__(y) <==> x-y """
  pass
 def __truediv__(self,*args):
  """ x.__truediv__(y) <==> x/y """
  pass
 def __trunc__(self,*args):
  """ __trunc__(x: SByte) -> SByte """
  pass
 def __xor__(self,*args):
  """ __xor__(x: SByte,y: SByte) -> SByte """
  pass
 denominator=None
 imag=None
 MaxValue=None
 MinValue=None
 numerator=None
 real=None

